# -*- coding: utf-8 -*-
# Copyright (c) 2020-2022, Vladimir Botka <vbotka@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
  name: lists_mergeby
  short_description: Merge two or more lists of dictionaries by a given attribute
  version_added: 2.0.0
  author: Vladimir Botka (@vbotka)
  description:
    - Merge two or more lists by attribute O(index). Optional parameters O(recursive) and O(list_merge)
      control the merging of the lists in values. The function merge_hash from ansible.utils.vars
      is used. To learn details on how to use the parameters O(recursive) and O(list_merge) see
      Ansible User's Guide chapter "Using filters to manipulate data" section "Combining
      hashes/dictionaries".
  positional: another_list, index
  options:
    _input:
      description: A list of dictionaries.
      type: list
      elements: dictionary
      required: true
    another_list:
      description: Another list of dictionaries. This parameter can be specified multiple times.
      type: list
      elements: dictionary
    index:
      description:
        - The dictionary key that must be present in every dictionary in every list that is used to
          merge the lists.
      type: string
      required: true
    recursive:
      description:
        - Should the combine recursively merge nested dictionaries (hashes).
        - "B(Note:) It does not depend on the value of the C(hash_behaviour) setting in C(ansible.cfg)."
      type: boolean
      default: false
    list_merge:
      description:
        - Modifies the behaviour when the dictionaries (hashes) to merge contain arrays/lists.
      type: string
      default: replace
      choices:
        - replace
        - keep
        - append
        - prepend
        - append_rp
        - prepend_rp
'''

EXAMPLES = '''
- name: Merge two lists
  ansible.builtin.debug:
    msg: >-
      {{ list1 | community.general.lists_mergeby(
                    list2,
                    'index',
                    recursive=True,
                    list_merge='append'
                 ) }}"
  vars:
    list1:
      - index: a
        value: 123
      - index: b
        value: 42
    list2:
      - index: a
        foo: bar
      - index: c
        foo: baz
  # Produces the following list of dictionaries:
  #   {
  #     "index": "a",
  #     "foo": "bar",
  #     "value": 123
  #   },
  #   {
  #     "index": "b",
  #     "value": 42
  #   },
  #   {
  #     "index": "c",
  #     "foo": "baz"
  #   }
'''

RETURN = '''
  _value:
    description: The merged list.
    type: list
    elements: dictionary
'''

from ansible.errors import AnsibleFilterError
from ansible.module_utils.six import string_types
from ansible.module_utils.common._collections_compat import Mapping, Sequence
from ansible.utils.vars import merge_hash

from collections import defaultdict
from operator import itemgetter


def list_mergeby(x, y, index, recursive=False, list_merge='replace'):
    ''' Merge 2 lists by attribute 'index'. The function merge_hash from ansible.utils.vars is used.
        This function is used by the function lists_mergeby.
    '''

    d = defaultdict(dict)
    for l in (x, y):
        for elem in l:
            if not isinstance(elem, Mapping):
                msg = "Elements of list arguments for lists_mergeby must be dictionaries. %s is %s"
                raise AnsibleFilterError(msg % (elem, type(elem)))
            if index in elem.keys():
                d[elem[index]].update(merge_hash(d[elem[index]], elem, recursive, list_merge))
    return sorted(d.values(), key=itemgetter(index))


def lists_mergeby(*terms, **kwargs):
    ''' Merge 2 or more lists by attribute 'index'. Optional parameters 'recursive' and 'list_merge'
        control the merging of the lists in values. The function merge_hash from ansible.utils.vars
        is used. To learn details on how to use the parameters 'recursive' and 'list_merge' see
        Ansible User's Guide chapter "Using filters to manipulate data" section "Combining
        hashes/dictionaries".

        Example:
        - debug:
            msg: "{{ list1|
                     community.general.lists_mergeby(list2,
                                                     'index',
                                                     recursive=True,
                                                     list_merge='append')|
                     list }}"
    '''

    recursive = kwargs.pop('recursive', False)
    list_merge = kwargs.pop('list_merge', 'replace')
    if kwargs:
        raise AnsibleFilterError("'recursive' and 'list_merge' are the only valid keyword arguments.")
    if len(terms) < 2:
        raise AnsibleFilterError("At least one list and index are needed.")

    # allow the user to do `[list1, list2, ...] | lists_mergeby('index')`
    flat_list = []
    for sublist in terms[:-1]:
        if not isinstance(sublist, Sequence):
            msg = ("All arguments before the argument index for community.general.lists_mergeby "
                   "must be lists. %s is %s")
            raise AnsibleFilterError(msg % (sublist, type(sublist)))
        if len(sublist) > 0:
            if all(isinstance(l, Sequence) for l in sublist):
                for item in sublist:
                    flat_list.append(item)
            else:
                flat_list.append(sublist)
    lists = flat_list

    if not lists:
        return []

    if len(lists) == 1:
        return lists[0]

    index = terms[-1]

    if not isinstance(index, string_types):
        msg = ("First argument after the lists for community.general.lists_mergeby must be string. "
               "%s is %s")
        raise AnsibleFilterError(msg % (index, type(index)))

    high_to_low_prio_list_iterator = reversed(lists)
    result = next(high_to_low_prio_list_iterator)
    for list in high_to_low_prio_list_iterator:
        result = list_mergeby(list, result, index, recursive, list_merge)

    return result


class FilterModule(object):
    ''' Ansible list filters '''

    def filters(self):
        return {
            'lists_mergeby': lists_mergeby,
        }



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: user
short_description: Resource module for User
description:
- Manage operations create and update of the resource User.
- Add a new user for Cisco DNA Center system.
- Update a user for Cisco DNA Center system.
version_added: '6.7.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  email:
    description: Email.
    type: str
  firstName:
    description: First Name.
    type: str
  lastName:
    description: Last Name.
    type: str
  password:
    description: Password.
    type: str
  roleList:
    description: Role id list.
    elements: str
    type: list
  userId:
    description: User Id.
    type: str
  username:
    description: Username.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for User and Roles AddUserAPI
  description: Complete reference of the AddUserAPI API.
  link: https://developer.cisco.com/docs/dna-center/#!add-user-api
- name: Cisco DNA Center documentation for User and Roles UpdateUserAPI
  description: Complete reference of the UpdateUserAPI API.
  link: https://developer.cisco.com/docs/dna-center/#!update-user-api
notes:
  - SDK Method used are
    user_and_roles.UserandRoles.add_user_ap_i,
    user_and_roles.UserandRoles.update_user_ap_i,

  - Paths used are
    post /dna/system/api/v1/user,
    put /dna/system/api/v1/user,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.user:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    email: string
    firstName: string
    lastName: string
    password: string
    roleList:
    - string
    username: string

- name: Update all
  cisco.dnac.user:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    email: string
    firstName: string
    lastName: string
    roleList:
    - string
    userId: string
    username: string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "message": "string",
      "userId": "string"
    }
"""



#!/usr/bin/python

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: command
author: "Petr Klima (@qaxi)"
short_description: Run commands on remote Cisco SMB devices
description:
  - Sends arbitrary commands to an Cisco SMB node and returns the results
    read from the device. This module includes an
    argument that will cause the module to wait for a specific condition
    before returning or timing out if the condition is not met.
options:
  commands:
    description:
      - List of commands to send to the remote Cisco SMB device over the
        configured provider. The resulting output from the command
        is returned. If the I(wait_for) argument is provided, the
        module is not returned until the condition is satisfied or
        the number of retries has expired.
    required: true
    type: list
    elements: str
  wait_for:
    description:
      - List of conditions to evaluate against the output of the
        command. The task will wait for each condition to be true
        before moving forward. If the conditional is not true
        within the configured number of retries, the task fails.
        See examples.
    type: list
    elements: str
  match:
    description:
      - The I(match) argument is used in conjunction with the
        I(wait_for) argument to specify the match policy.  Valid
        values are C(all) or C(any).  If the value is set to C(all)
        then all conditionals in the wait_for must be satisfied.  If
        the value is set to C(any) then only one of the values must be
        satisfied.
    default: all
    type: str
    choices: ['any', 'all']
  retries:
    description:
      - Specifies the number of retries a command should by tried
        before it is considered failed. The command is run on the
        target device every retry and evaluated against the
        I(wait_for) conditions.
    default: 10
    type: int
  interval:
    description:
      - Configures the interval in seconds to wait between retries
        of the command. If the command does not pass the specified
        conditions, the interval indicates how long to wait before
        trying the command again.
    default: 1
    type: int
'''

EXAMPLES = """
- name: Run command on remote devices
  community.ciscosmb.command:
    commands: show version

- name: Run command and check to see if output contains PID
  community.ciscosmb.command:
    commands: show inventory
    wait_for: result[0] contains PID

- name: Run multiple commands on remote nodes
  community.ciscosmb.command:
    commands:
      - show version
      - show system

- name: Run multiple commands and evaluate the output
  community.ciscosmb.command:
    commands:
      - show version
      - show system
    wait_for:
      - result[0] contains Active-image
      - result[1] contains "System Up Time"
"""

RETURN = """
stdout:
  description: The set of responses from the commands.
  returned: always apart from low level errors (such as action plugin)
  type: list
  sample: ['...', '...']
stdout_lines:
  description: The value of stdout split into a list.
  returned: always apart from low level errors (such as action plugin)
  type: list
  sample: [['...', '...'], ['...'], ['...']]
failed_conditions:
  description: The list of conditionals that have failed.
  returned: failed
  type: list
  sample: ['...', '...']
"""

import time

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.parsing import Conditional
from ansible_collections.community.ciscosmb.plugins.module_utils.ciscosmb import run_commands
from ansible_collections.community.ciscosmb.plugins.module_utils.ciscosmb import ciscosmb_argument_spec
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.six import string_types


def to_lines(stdout):
    for item in stdout:
        if isinstance(item, string_types):
            item = str(item).split('\n')
        yield item


def main():
    """main entry point for module execution
    """
    argument_spec = dict(
        commands=dict(type='list', elements='str', required=True),

        wait_for=dict(type='list', elements='str'),
        match=dict(default='all', choices=['all', 'any']),

        retries=dict(default=10, type='int'),
        interval=dict(default=1, type='int')
    )

    argument_spec.update(ciscosmb_argument_spec)

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=False)

    result = {'changed': False}

    wait_for = module.params['wait_for'] or list()
    conditionals = [Conditional(c) for c in wait_for]

    retries = module.params['retries']
    interval = module.params['interval']
    match = module.params['match']

    while retries > 0:
        responses = run_commands(module, module.params['commands'])

        for item in list(conditionals):
            if item(responses):
                if match == 'any':
                    conditionals = list()
                    break
                conditionals.remove(item)

        if not conditionals:
            break

        time.sleep(interval)
        retries -= 1

    if conditionals:
        failed_conditions = [item.raw for item in conditionals]
        msg = 'One or more conditional statements have not been satisfied'
        module.fail_json(msg=msg, failed_conditions=failed_conditions)

    result.update({
        'changed': False,
        'stdout': responses,
        'stdout_lines': list(to_lines(responses))
    })

    module.exit_json(**result)


if __name__ == '__main__':
    main()



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# (c) 2018 Red Hat Inc.
#
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible.module_utils.connection import Connection


def get(module, path=None, content=None, fields=None, output="json"):
    if path is None:
        raise ValueError("path value must be provided")
    if content:
        path += "?" + "content=%s" % content
    if fields:
        path += "?" + "field=%s" % fields

    accept = None
    if output == "xml":
        accept = "application/yang-data+xml"

    connection = Connection(module._socket_path)
    return connection.send_request(None, path=path, method="GET", accept=accept)


def edit_config(module, path=None, content=None, method="GET", format="json"):
    if path is None:
        raise ValueError("path value must be provided")

    content_type = None
    if format == "xml":
        content_type = "application/yang-data+xml"

    connection = Connection(module._socket_path)
    return connection.send_request(content, path=path, method=method, content_type=content_type)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_wireless_ssids_identity_psks
short_description: Resource module for networks _wireless _ssids _identitypsks
description:
- Manage operations create, update and delete of the resource networks _wireless _ssids _identitypsks.
- Create an Identity PSK.
- Delete an Identity PSK.
- Update an Identity PSK.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  expiresAt:
    description: Timestamp for when the Identity PSK expires. Will not expire if left
      blank.
    type: str
  groupPolicyId:
    description: The group policy to be applied to clients.
    type: str
  identityPskId:
    description: IdentityPskId path parameter. Identity psk ID.
    type: str
  name:
    description: The name of the Identity PSK.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  number:
    description: Number path parameter.
    type: str
  passphrase:
    description: The passphrase for client authentication. If left blank, one will be
      auto-generated.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for wireless createNetworkWirelessSsidIdentityPsk
  description: Complete reference of the createNetworkWirelessSsidIdentityPsk API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-network-wireless-ssid-identity-psk
- name: Cisco Meraki documentation for wireless deleteNetworkWirelessSsidIdentityPsk
  description: Complete reference of the deleteNetworkWirelessSsidIdentityPsk API.
  link: https://developer.cisco.com/meraki/api-v1/#!delete-network-wireless-ssid-identity-psk
- name: Cisco Meraki documentation for wireless updateNetworkWirelessSsidIdentityPsk
  description: Complete reference of the updateNetworkWirelessSsidIdentityPsk API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-wireless-ssid-identity-psk
notes:
  - SDK Method used are
    wireless.Wireless.create_network_wireless_ssid_identity_psk,
    wireless.Wireless.delete_network_wireless_ssid_identity_psk,
    wireless.Wireless.update_network_wireless_ssid_identity_psk,

  - Paths used are
    post /networks/{networkId}/wireless/ssids/{number}/identityPsks,
    delete /networks/{networkId}/wireless/ssids/{number}/identityPsks/{identityPskId},
    put /networks/{networkId}/wireless/ssids/{number}/identityPsks/{identityPskId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_wireless_ssids_identity_psks:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    expiresAt: '2018-02-11T00:00:00.090210Z'
    groupPolicyId: '101'
    id: '1284392014819'
    name: Sample Identity PSK
    networkId: string
    number: string
    passphrase: secret

- name: Update by id
  cisco.meraki.networks_wireless_ssids_identity_psks:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    expiresAt: '2018-02-11T00:00:00.090210Z'
    groupPolicyId: '101'
    id: '1284392014819'
    identityPskId: string
    name: Sample Identity PSK
    networkId: string
    number: string
    passphrase: secret

- name: Delete by id
  cisco.meraki.networks_wireless_ssids_identity_psks:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: absent
    identityPskId: string
    networkId: string
    number: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Route_maps parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_route_map_set_aspath_prepend(config_data):
    el = config_data["entries"]
    command = "set as-path prepend "
    c = el["set"]["as_path"]["prepend"]
    if c.get("last_as"):
        command += "last-as " + str(c["last_as"])
    if c.get("as_number"):
        num = " ".join(c["as_number"].split(","))
        command += num
    return command


def _tmplt_route_map_set_aspath_match(config_data):
    el = config_data["entries"]
    command = "set as-path match all replacement "
    c = el["set"]["as_path"]["match"]
    if c.get("none"):
        command += "none"
    if c.get("as_number"):
        num = str(c["as_number"])
        command += num
    return command


def _tmplt_route_map_extcommunity_lbw(config_data):
    config_data = config_data["entries"]["set"]["extcommunity"]["lbw"]
    command = "set extcommunity lbw "
    if config_data.get("aggregate"):
        command += "aggregate " + config_data["value"]
    if not config_data.get("aggregate") and config_data.get("value"):
        command += config_data["value"]
    if config_data.get("divide"):
        command += "divide " + config_data["divide"]
    return command


def _tmplt_route_map_extcommunity_rt(config_data):
    config_data = config_data["entries"]["set"]["extcommunity"]["rt"]
    command = "set extcommunity rt " + config_data["vpn"]
    if config_data.get("additive"):
        command += " additive"
    if config_data.get("delete"):
        command += " delete"
    return command


def _tmplt_route_maps_subroutemap(config_data):
    command = ""
    if config_data["entries"].get("sub_route_map"):
        command = "sub-route-map " + config_data["entries"]["sub_route_map"]["name"]
    if config_data["entries"]["sub_route_map"].get("invert_result"):
        command += " invert-result"
    return command


def _tmplt_route_map_extcommunity_soo(config_data):
    config_data = config_data["entries"]["set"]["extcommunity"]["soo"]
    command = "set extcommunity soo " + config_data["vpn"]
    if config_data.get("additive"):
        command += " additive"
    if config_data.get("delete"):
        command += " delete"
    return command


def _tmplt_route_map_ip(config_data):
    config_data = config_data["entries"]["set"]
    if config_data.get("ip"):
        command = "set ip next-hop "
        k = "ip"
    elif config_data.get("ipv6"):
        command = "set ip next-hop "
        k = "ipv6"
    if config_data[k].get("address"):
        command += config_data[k]["address"]
    elif config_data[k].get("unchanged"):
        command += "unchanged"
    elif config_data[k].get("peer_address"):
        command += "peer-address"
    return command


def _tmplt_route_maps_metric(config_data):
    config_data = config_data["entries"]["set"]["metric"]
    command = "set metric"
    if config_data.get("value"):
        command += " " + config_data["value"]
    if config_data.get("add"):
        command += " +" + config_data["add"]
    if config_data.get("igp_param"):
        command += " " + config_data["igp_param"]
    return command


def _tmplt_route_maps_nexthop(config_data):
    config_data = config_data["entries"]["set"]["nexthop"]
    command = "set next-hop igp-metric "
    if config_data.get("max_metric"):
        command += "max-metric"
    if config_data.get("value"):
        command += config_data["value"]
    return command


def _tmplt_route_map_match_aggregator_role(config_data):
    config_data = config_data["entries"]["match"]["aggregator_role"]
    command = "match aggregator-role contributor"
    if config_data.get("route_map"):
        command += " aggregate-attributes " + config_data["route_map"]
    return command


def _tmplt_route_map_match_aspath(config_data):
    config_data = config_data["entries"]["match"]["as_path"]
    command = "match as-path "
    if config_data.get("length"):
        command += "length " + config_data["length"]
    if config_data.get("path_list"):
        command += config_data["path_list"]
    return command


def _tmplt_route_map_match_invert_aggregator_role(config_data):
    config_data = config_data["entries"]["match"]["invert_result"]["aggregate_role"]
    command = "match invert-result as-path aggregate-role contributor"
    if config_data.get("route_map"):
        command += " aggregator-attributes " + config_data["route_map"]
    return command


def _tmplt_route_map_match_invert_aspath(config_data):
    config_data = config_data["entries"]["match"]["invert_result"]["as_path"]
    command = "match invert-result as-path "
    if config_data.get("length"):
        command += "length " + config_data["length"]
    if config_data.get("path_list"):
        command += config_data["path_list"]
    return command


def _tmplt_route_map_match_ip_address(config_data):
    command = ""
    config_data = config_data["entries"]["match"]["ip"]
    if config_data.get("address"):
        config_data = config_data["address"]
        command = "match ip address "
        if config_data.get("dynamic"):
            command += "dynamic"
        if config_data.get("access_list"):
            command += "access-list " + config_data["access_list"]
        if config_data.get("prefix_list"):
            command += "prefix-list " + config_data["prefix_list"]
    return command


def _tmplt_route_map_match_ipv6_address(config_data):
    command = ""
    config_data = config_data["entries"]["match"]["ipv6"]
    if config_data.get("address"):
        config_data = config_data["address"]
        command = "match ipv6 address "
        if config_data.get("dynamic"):
            command += "dynamic"
        if config_data.get("access_list"):
            command += "access-list " + config_data["access_list"]
        if config_data.get("prefix_list"):
            command += "prefix-list " + config_data["prefix_list"]
    return command


def _tmplt_route_map_match_ip(config_data):
    command = ""
    config_data = config_data["entries"]["match"]["ip"]
    if "address" not in config_data:
        command = "match ip "
        if config_data.get("next_hop"):
            command += "next-hop prefix-list " + config_data["next_hop"]
        elif config_data.get("resolved_next_hop"):
            command += "resolved-next-hop prefix-list " + config_data["resolved_next_hop"]
    return command


def _tmplt_route_map_match_ipv6(config_data):
    command = ""
    config_data = config_data["entries"]["match"]["ipv6"]
    if "address" not in config_data:
        command = "match ipv6 "
        if config_data.get("next_hop"):
            command += "next-hop prefix-list " + config_data["next_hop"]
        elif config_data.get("resolved_next_hop"):
            command += "resolved-next-hop prefix-list " + config_data["resolved_next_hop"]
    return command


def _tmplt_route_maps_match_metric(config_data):
    config_data = config_data["entries"]["match"]["metric"]
    command = "match metric"
    if config_data.get("value"):
        command += " " + config_data["value"]
    return command


class Route_mapsTemplate(NetworkTemplate):
    def __init__(self, lines=None):
        super(Route_mapsTemplate, self).__init__(lines=lines, tmplt=self)

    # fmt: off
    PARSERS = [
        {
            "name": "route_map.entries",
            "getval": re.compile(
                r"""
                \s*route-map
                \s+(?P<map_name>\S+)
                \s+(?P<action>deny|permit)
                \s+(?P<seq>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "route-map {{ route_map }} {{ entries.action }} {{ entries.sequence }}",
            "compval": "entries",
            "result": {
                "route_map": "{{ map_name }}",
                "entries": [
                    {
                        "action": "{{ action }}",
                        "sequence": "{{ seq }}",
                    },
                ],
            },
            "shared": True,
        },
        {
            "name": "route_map.action",
            "getval": re.compile(
                r"""
                \s*route-map
                \s+(?P<map_name>\S+)
                \s+(?P<action>deny|permit)
                $""",
                re.VERBOSE,
            ),
            "setval": "route-map {{ route_map }} {{ entries.action }}",
            "compval": "entries.action",
            "result": {
                "route_map": "{{ map_name }}",
                "entries": [
                    {
                        "action": "{{ action }}",
                    },
                ],
            },
            "shared": True,
        },
        {
            "name": "route_map.name",
            "getval": re.compile(
                r"""
                \s*route-map
                \s+(?P<map_name>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "route-map {{ route_map }}",
            "compval": "route_map",
            "result": {
                "route_map": "{{ map_name }}",
            },
            "shared": True,
        },
        {
            "name": "route_map.statement.entries",
            "getval": re.compile(
                r"""
                \s*route-map
                \s+(?P<map_name>\S+)
                \s+statement
                \s+(?P<statement>\S+)
                \s+(?P<action>deny|permit)
                \s+(?P<seq>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "route-map {{ route_map }} statement {{ entries.statement }}" +
                      " {{ entries.action }} {{ entries.sequence }}",
            "compval": "entries.statement",
            "result": {
                "route_map": "{{ map_name }}",
                "entries": [
                    {
                        "statement": "{{ statement }}",
                        "action": "{{ action }}",
                        "sequence": "{{ seq }}",
                    },
                ],
            },
            "shared": True,
        },
        {
            "name": "route_map.statement.action",
            "getval": re.compile(
                r"""
                \s*route-map
                \s+(?P<map_name>\S+)
                \s+statement
                \s+(?P<statement>\S+)
                \s+(?P<action>deny|permit)
                $""",
                re.VERBOSE,
            ),
            "setval": "route-map {{ route_map }} statement {{ entries.statement }} {{ entries.action }}",
            "compval": "entries.statement",
            "result": {
                "route_map": "{{ map_name }}",
                "entries": [
                    {
                        "statement": "{{ statement }}",
                        "action": "{{ action }}",
                    },
                ],
            },
            "shared": True,
        },
        {
            "name": "route_map.statement.name",
            "getval": re.compile(
                r"""
                \s*route-map
                \s+(?P<map_name>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "route-map {{ route_map }} statement {{ entries.statement }}",
            "compval": "entries.statement",
            "result": {
                "route_map": "{{ map_name }}",
                "entries": [
                    {
                        "statement": "{{ statement }}",
                    },
                ],
            },
            "shared": True,
        },
        {
            "name": "continue",
            "getval": re.compile(
                r"""
                \s*continue
                \s+(?P<num>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "continue {{ entries.continue_sequence }}",
            "compval": "entries.continue_sequence",
            "result": {
                "entries": [
                    {
                        "continue_sequence": "{{ num }}",
                    },
                ],
            },
        },
        {
            "name": "route_map.copy",
            "getval": re.compile(
                r"""
                \s*route-map
                \s+(?P<map_name>\S+)
                \s+copy
                \s+(?P<name>\S+)
                \s*(?P<overwrite>overwrite)*
                $""",
                re.VERBOSE,
            ),
            "setval": "route-map {{ route_map }} copy {{ entries.source.source_map_name }}" +
                      "{{ (' ' + overwrite) if overwrite is defined }}",
            "compval": "entries.source",
            "result": {
                "route_map": "{{ map_name }}",
                "entries": [
                    {
                        "source": {
                            "action": "copy",
                            "source_map_name": "{{ name }}",
                            "overwrite": "{{ True if overwrite is defined }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "route_map.rename",
            "getval": re.compile(
                r"""
                \s*route-map
                \s+(?P<map_name>\S+)
                \s+rename
                \s+(?P<name>\S+)
                \s*(?P<overwrite>overwrite)*
                $""",
                re.VERBOSE,
            ),
            "setval": "route-map {{ route_map }} rename {{ entries.source.source_map_name }}" +
                      "{{ (' ' + overwrite) if overwrite is defined }}",
            "compval": "entries.source",
            "result": {
                "route_map": "{{ map_name }}",
                "entries": [
                    {
                        "source": {
                            "action": "rename",
                            "source_map_name": "{{ name }}",
                            "overwrite": "{{ True if overwrite is defined }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "description",
            "getval": re.compile(
                r"""
                \s*description
                \s+(?P<desc>.+)
                $""",
                re.VERBOSE,
            ),
            "setval": "description {{ entries.description }}",
            "compval": "entries.description",
            "remval": "description",
            "result": {
                "entries": [
                    {
                        "description": "{{ desc }}",
                    },
                ],
            },
        },
        {
            "name": "sub_route_map",
            "getval": re.compile(
                r"""
                \s*sub-route-map
                \s*(?P<invert>invert-result)*
                \s+(?P<map>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_maps_subroutemap,
            "compval": "entries.sub_route_map",
            "result": {
                "entries": [
                    {
                        "sub_route_map": {
                            "name": "{{ map }}",
                            "invert_result": "{{ True if invert is defined }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "set.as_path.prepend",
            "getval": re.compile(
                r"""
                \s*set\s+as-path\sprepend
                \s*(?P<lastas>last-as .+)*
                \s*(?P<as>[^a-zA-Z]+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_set_aspath_prepend,
            "compval": "entries.set.as_path.prepend",
            "result": {
                "entries": [
                    {
                        "set": {
                            "as_path": {
                                "prepend": {
                                    "last_as": "{{ lastas.split(" ")[1] if lastas is defined }}",
                                    "as_number": "{{ ','.join(as.split(' ')) if as is defined }}",
                                },
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.as_path.match",
            "getval": re.compile(
                r"""
                \s*set\s+as-path\smatch\sall\replacement
                \s*(?P<none>none)*
                \s*(?P<as>.+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_set_aspath_match,
            "compval": "entries.set.as_path.match",
            "result": {
                "entries": [
                    {
                        "set": {
                            "as_path": {
                                "match": {
                                    "none": "{{ True if none is defined }}",
                                    "as_number": "{{ as }}",
                                },
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.bgp",
            "getval": re.compile(
                r"""
                \s*set\sbgp\sbestpath\sas-path\sweight
                \s+(?P<weight>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "set bgp bestpath as-path weight {{ entries.set.bgp }}",
            "compval": "entries.set.bgp",
            "result": {
                "entries": [
                    {
                        "set": {
                            "bgp": "{{ weight }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "set.community.graceful_shutdown",
            "getval": re.compile(
                r"""
                \s*set\scommunity\sGSHUT
                *$""",
                re.VERBOSE,
            ),
            "setval": "set community GSHUT",
            "compval": "entries.set.graceful_shutdown",
            "result": {
                "entries": [
                    {
                        "set": {
                            "graceful_shutdown": "{{ True }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "set.community.none",
            "getval": re.compile(
                r"""
                \s*set\scommunity\snone
                *$""",
                re.VERBOSE,
            ),
            "setval": "set community none",
            "compval": "entries.set.none",
            "result": {
                "entries": [
                    {
                        "set": {
                            "none": "{{ True }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "set.community.number",
            "getval": re.compile(
                r"""
                \s*set\scommunity
                \s+(?P<num>\d+\s*)+
                \s*(?P<action>additive|delete)*
                \s*(?P<donot>local-as|no-advertise|no-export)*
                $""",
                re.VERBOSE,
            ),
            "setval": "set community {{ entries.set.community.number }}" +
                      "{{ (' ' + action) if action is defined }}{{  (' ' + donot) if donot is defined }}",
            "compval": "entries.set.community",
            "result": {
                "entries": [
                    {
                        "set": {
                            "community": {
                                "number": "{{ num }}",
                                "additive": "{{ True if action == 'additive' }}",
                                "delete": "{{ True if action == 'delete' }}",
                                '{{ "donot" }}': "{{ True if donot is defined }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.community.list",
            "getval": re.compile(
                r"""
                \s*set\scommunity\s+community-list
                \s+(?P<name>\S+\s*)+
                \s*(?P<action>additive|delete)*
                \s*(?P<donot>local-as|no-advertise|no-export)*
                $""",
                re.VERBOSE,
            ),
            "setval": "set community {{ entries.set.community.name }}" +
                      "{{ (' ' + action) if action is defined }}{{  (' ' + donot) if donot is defined }}",
            "compval": "entries.set.community",
            "result": {
                "entries": [
                    {
                        "set": {
                            "community": {
                                "community_list": "{{ name }}",
                                "additive": "{{ True if action == 'additive' }}",
                                "delete": "{{ True if action == 'delete' }}",
                                '{{ "donot" }}': "{{ True if donot is defined }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.community.internet",
            "getval": re.compile(
                r"""
                \s*set\scommunity\s+internet
                \s*(?P<action>additive|delete)*
                \s*(?P<donot>local-as|no-advertise|no-export)*
                $""",
                re.VERBOSE,
            ),
            "setval": "set community internet" +
                      "{{ (' ' + action) if action is defined }}{{  (' ' + donot) if donot is defined }}",
            "compval": "entries.set.community",
            "result": {
                "entries": [
                    {
                        "set": {
                            "community": {
                                "internet": "{{ true }}",
                                "additive": "{{ True if action == 'additive' }}",
                                "delete": "{{ True if action == 'delete' }}",
                                '{{ "donot" }}': "{{ True if donot is defined }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.distance",
            "getval": re.compile(
                r"""
                \s*set\sdistance
                \s+(?P<distance>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "set distance {{ entries.set.distance }}",
            "compval": "entries.set.distance",
            "result": {
                "entries": [
                    {
                        "set": {
                            "distance": "{{ distance }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "set.evpn",
            "getval": re.compile(
                r"""
                \s*set\sevpn\snext-hop\sunchanged
                $""",
                re.VERBOSE,
            ),
            "setval": "set evpn next-hop unchanged",
            "compval": "entries.set.evpn",
            "result": {
                "entries": [
                    {
                        "set": {
                            "evpn": "{{ True }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "set.extcommunity.lbw",
            "getval": re.compile(
                r"""
                \s*set\sextcommunity\slbw
                \s*(?P<agg>aggregate)*
                \s*(?P<divide>divide\s\S+)*
                \s*(?P<value>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_extcommunity_lbw,
            "compval": "entries.set.extcommunity.lbw",
            "result": {
                "entries": [
                    {
                        "set": {
                            "extcommunity": {
                                "lbw": {
                                    "value": "{{ value if value is defined }}",
                                    "aggregate": "{{ True if agg is defined }}",
                                    "divide": "{{ divide.split(" ")[1] if divide is defined }}",
                                },
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.extcommunity.none",
            "getval": re.compile(
                r"""
                \s*set\sextcommunity\snone
                $""",
                re.VERBOSE,
            ),
            "setval": "{{ set extcommunity none }}",
            "compval": "entries.set.extcommunity.none",
            "result": {
                "entries": [
                    {
                        "set": {
                            "extcommunity": {
                                "none": "{{ True }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.extcommunity.rt",
            "getval": re.compile(
                r"""
                \s*set\sextcommunity\srt
                \s+(?P<asn>\S+)
                \s*(?P<action_add>additive)*
                \s*(?P<action_del>delete)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_extcommunity_rt,
            "compval": "entries.set.extcommunity.rt",
            "result": {
                "entries": [
                    {
                        "set": {
                            "extcommunity": {
                                "rt": {
                                    "vpn": "{{ asn }}",
                                    "additive": "{{ True if action_add is defined }}",
                                    "delete": "{{ True if action_del is defined }}",
                                },
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.extcommunity.soo",
            "getval": re.compile(
                r"""
                \s*set\sextcommunity\ssoo
                \s+(?P<asn>\S+)
                \s+(?P<action>additive|delete)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_extcommunity_soo,
            "compval": "entries.set.extcommunity.soo",
            "result": {
                "entries": [
                    {
                        "set": {
                            "extcommunity": {
                                "soo": {
                                    "vpn": "{{ asn }}",
                                    "additive": "{{ True if action == 'additive' }}",
                                    "delete": "{{ True if action == 'delete' }}",
                                },
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.ip",
            "getval": re.compile(
                r"""
                \s*set\sip\snext-hop
                \s+(?P<attr>peer-address|unchanged|[\d\.]+)
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_ip,
            "compval": "entries.set.ip",
            "result": {
                "entries": [
                    {
                        "set": {
                            "ip": {
                                "unchanged": "{{ True if attr == 'unchanged' }}",
                                "peer_address": "{{ True if attr == 'peer-address' }}",
                                "address": "{{ attr if attr != 'unchanged' and attr != 'peer-address' }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.ipv6",
            "getval": re.compile(
                r"""
                \s*set\sipv6\snext-hop
                \s+(?P<attr>peer-address|unchanged|[\d\.]+)
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_ip,
            "compval": "entries.set.ipv6",
            "result": {
                "entries": [
                    {
                        "set": {
                            "ipv6": {
                                "unchanged": "{{ True if attr == 'unchanged' }}",
                                "peer_address": "{{ True if attr == 'peer-address' }}",
                                "address": "{{ attr if attr != 'unchanged' and attr != 'peer-address' }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.isis",
            "getval": re.compile(
                r"""
                \s*set\sisis\slevel
                \s+(?P<level>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "set isis level {{ entries.set.isis_level }}",
            "compval": "entries.set.isis_level",
            "result": {
                "entries": [
                    {
                        "set": {
                            "isis_level": "{{ level }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "set.local_pref",
            "getval": re.compile(
                r"""
                \s*set\slocal-preference
                \s+(?P<as>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "set local-preference {{ entries.set.local_preference }}",
            "compval": "entries.set.local_preference",
            "result": {
                "entries": [
                    {
                        "set": {
                            "local_preference": "{{ as }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "set.metric.value",
            "getval": re.compile(
                r"""
                \s*set\smetric
                \s*(?P<val>\d+)*
                \s*(?P<operation>\+\S+)*
                \s*(?P<param>igp-metric|igp-nexthop-cost)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_maps_metric,
            "compval": "entries.set.metric",
            "result": {
                "entries": [
                    {
                        "set": {
                            "metric": {
                                "value": "{{ val }}",
                                "add": "{{ operation.strip('+') }}",
                                "igp_param": "{{ param }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.metric_type",
            "getval": re.compile(
                r"""
                \s*set\smetric-type
                \s+(?P<type>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "set metric-type {{ entries.set.metric_type }}",
            "compval": "entries.set.local_preference",
            "result": {
                "entries": [
                    {
                        "set": {
                            "metric_type": "{{ type }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "set.nexthop",
            "getval": re.compile(
                r"""
                \s*set\snext-hop\sigp-metric
                \s*(?P<hop>\d+)*
                \s*(?P<max>max-metric)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_maps_nexthop,
            "compval": "entries.set.nexthop",
            "result": {
                "entries": [
                    {
                        "set": {
                            "nexthop": {
                                "value": "{{ hop if hop is defined }}",
                                "max_metric": "{{ True if max is defined }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "set.origin",
            "getval": re.compile(
                r"""
                \s*set\sorigin
                \s+(?P<param>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "set origin {{ entries.set.origin }}",
            "compval": "entries.set.origin",
            "result": {
                "entries": [
                    {
                        "set": {
                            "origin": "{{ param }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "set.segment_index",
            "getval": re.compile(
                r"""
                \s*set\ssegment-index
                \s+(?P<index>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "set segment-index {{ entries.set.segment_index }}",
            "compval": "entries.set.segment_index",
            "result": {
                "entries": [
                    {
                        "set": {
                            "segment_index": "{{ index }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "set.tag",
            "getval": re.compile(
                r"""
                \s*set\stag
                \s+(?P<val>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "set tag {{ entries.set.tag }}",
            "compval": "entries.set.tag",
            "result": {
                "entries": [
                    {
                        "set": {
                            "tag": "{{ val }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "set.weight",
            "getval": re.compile(
                r"""
                \s*set\sweight
                \s+(?P<val>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "set local-preference {{ entries.set.weight }}",
            "compval": "entries.set.weight",
            "result": {
                "entries": [
                    {
                        "set": {
                            "weight": "{{ val }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "match.aggregate_role",
            "getval": re.compile(
                r"""
                \s*match\s+aggregator-role\scontributor
                \s*(?P<map>aggregate-attributes \S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_match_aggregator_role,
            "compval": "entries.match.aggregator_role",
            "result": {
                "entries": [
                    {
                        "match": {
                            "aggregator_role": {
                                "contributor": "{{ True if map is not defined }}",
                                "route_map": "{{ map.split(" ")[1] }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.as",
            "getval": re.compile(
                r"""
                \s*match\s+as
                \s+(?P<as>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": "match as {{ entries.match.as }}",
            "compval": "entries.match.as",
            "result": {
                "entries": [
                    {
                        "match": {
                            "as": "{{ as }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "match.as_path",
            "getval": re.compile(
                r"""
                \s*match\s+as-path
                \s*(?P<len>length .+)*
                \s*(?P<path>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_match_aspath,
            "compval": "entries.match.as_path.prepend",
            "result": {
                "entries": [
                    {
                        "match": {
                            "as_path": {
                                "path_list": "{{ path }}",
                                "length": "{{ len[7:] }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.community.instances",
            "getval": re.compile(
                r"""
                \s*match\scommunity\sinstances
                \s+(?P<inst>.+)
                *$""",
                re.VERBOSE,
            ),
            "setval": "match community instances {{ entries.match.community.instances }}",
            "compval": "entries.match.community.instances",
            "result": {
                "entries": [
                    {
                        "match": {
                            "community": {
                                "instances": "{{ inst }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.community.list",
            "getval": re.compile(
                r"""
                \s*match\scommunity
                \s+(?P<comm>.+\s)
                \s*(?P<mat>exact-match)*
                $""",
                re.VERBOSE,
            ),
            "setval": "match community {{ entries.match.community.community_list }}{{  (' exact-match') if mat is defined }}",
            "compval": "entries.match.community",
            "result": {
                "entries": [
                    {
                        "match": {
                            "community": {
                                "community_list": "{{ comm.strip() }}",
                                "exact_match": "{{ True if mat is defined }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.extcommunity",
            "getval": re.compile(
                r"""
                \s*match\sextcommunity
                \s+(?P<list>.+\s*)
                \s*(?P<mat>exact-match)*
                $""",
                re.VERBOSE,
            ),
            "setval": "match extcommunity {{ entries.match.extcommunity.community_list }}{{  (' exact-match') if mat is defined }}",
            "compval": "entries.match.extcommunity",
            "result": {
                "entries": [
                    {
                        "match": {
                            "extcommunity": {
                                "community_list": "{{ list }}",
                                "exact_match": "{{ True if mat is defined }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.invert.aggregate_role",
            "getval": re.compile(
                r"""
                \s*match\sinvert-result\saggregate-role\scontributor
                \s*(?P<map>aggregate-attributes \S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_match_invert_aggregator_role,
            "compval": "entries.match.invert_result.aggregate_role",
            "result": {
                "entries": [
                    {
                        "match": {
                            "invert_result": {
                                "aggregate_role": {
                                    "contributor": "{{ True if map is not defined }}",
                                    "route_map": "{{ map.split(" ")[1] }}",
                                },
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.invert.as_path",
            "getval": re.compile(
                r"""
                \s*match\sinvert-result\sas-path
                \s*(?P<len>length .+)*
                \s*(?P<path>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_match_invert_aspath,
            "compval": "entries.match.invert_result.as_path.prepend",
            "result": {
                "entries": [
                    {
                        "match": {
                            "invert_result": {
                                "as_path": {
                                    "path_list": "{{ path }}",
                                    "length": "{{ len[7:] }}",
                                },
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.invert.community.instances",
            "getval": re.compile(
                r"""
                \s*match\sinvert-result\scommunity\sinstances
                \s+(?P<inst>.+)
                *$""",
                re.VERBOSE,
            ),
            "setval": "match invert-result community instances {{ entries.match.community.instances }}",
            "compval": "entries.match.invert_result.community.instances",
            "result": {
                "entries": [
                    {
                        "match": {
                            "invert_result": {
                                "community": {
                                    "instances": "{{ inst }}",
                                },
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.invert.community.list",
            "getval": re.compile(
                r"""
                \s*match\sinvert-result\scommunity
                \s+(?P<list>.+\s*)
                \s*(?P<mat>exact-match)
                *$""",
                re.VERBOSE,
            ),
            "setval": "match invert-result community {{ entries.match.community.community_list }}{{  (' exact-match') if mat is defined }}",
            "compval": "entries.match.invert_result.community",
            "result": {
                "entries": [
                    {
                        "match": {
                            "invert_result": {
                                "community": {
                                    "community_list": "{{ list }}",
                                    "exact_match": "{{ True if mat is defined }}",
                                },
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.invert.extcommunity",
            "getval": re.compile(
                r"""
                \s*match\sinvert-result\sextcommunity
                \s+(?P<list>.+\s*)
                \s*(?P<mat>exact-match)
                *$""",
                re.VERBOSE,
            ),
            "setval": "match invert-result extcommunity" +
                      " {{ entries.match.extcommunity.community_list }}{{ (' exact-match') if mat is defined }}",
            "compval": "entries.match.invert_result.extcommunity",
            "result": {
                "entries": [
                    {
                        "match": {
                            "invert_result": {
                                "extcommunity": {
                                    "community_list": "{{ list }}",
                                    "exact_match": "{{ True if mat is defined }}",
                                },
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.interface",
            "getval": re.compile(
                r"""
                \s*match\sinterface
                \s+(?P<int>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "match interface {{ entries.match.interface }}",
            "compval": "entries.match.interface",
            "result": {
                "entries": [
                    {
                        "match": {
                            "interface": "{{ int }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "match.ipaddress",
            "getval": re.compile(
                r"""
                \s*match\sip\saddress
                \s*(?P<dyn>dynamic)*
                \s+(?P<attr>\S+\s\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_match_ip_address,
            "compval": "entries.match.ip.address",
            "shared": True,
            "result": {
                "entries": [
                    {
                        "match": {
                            "ip": {
                                "address": {
                                    "dynamic": "{{ True if dynamic is defined }}",
                                    "access_list": '{{ attr.split(" ")[1] if attr.split(" ")[0] == "access-list" }}',
                                    "prefix_list": '{{ attr.split(" ")[1] if attr.split(" ")[0] == "prefix-list" }}',
                                },
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.ip",
            "getval": re.compile(
                r"""
                \s*match\sip
                \s+(?P<param>next-hop|resolved-next-hop)
                \s+prefix-list
                \s+(?P<prefix>\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_match_ip,
            "compval": "entries.match.ip",
            "result": {
                "entries": [
                    {
                        "match": {
                            "ip": {
                                "next_hop": "{{ prefix if param == 'next-hop' }}",
                                "resolved_next_hop": "{{ prefix if param == 'resolved-next-hop' }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.ipv6address",
            "getval": re.compile(
                r"""
                \s*match\sipv6\saddress
                \s*(?P<dyn>dynamic)*
                \s+(?P<attr>\S+\s\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_match_ipv6_address,
            "compval": "entries.match.ipv6.address",
            "shared": True,
            "result": {
                "entries": [
                    {
                        "match": {
                            "ipv6": {
                                "address": {
                                    "dynamic": "{{ True if dynamic is defined }}",
                                    "access_list": '{{ attr.split(" ")[1] if attr.split(" ")[0] == "access-list" }}',
                                    "prefix_list": '{{ attr.split(" ")[1] if attr.split(" ")[0] == "prefix-list" }}',
                                },
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.ipv6",
            "getval": re.compile(
                r"""
                \s*match\sipv6
                \s*(?P<param>next-hop|resolved-next-hop)
                \s+prefix-list
                \s+(?P<prefix>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_map_match_ipv6,
            "compval": "entries.match.ipv6",
            "result": {
                "entries": [
                    {
                        "match": {
                            "ipv6": {
                                "next_hop": "{{ prefix if param == 'next-hop' }}",
                                "resolved_next_hop": "{{ prefix if param == 'resolved-next-hop' }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.largecommunity",
            "getval": re.compile(
                r"""
                \s*match\slarge-community
                \s+(?P<list>.+\s*)
                \s*(?P<mat>exact-match)
                *$""",
                re.VERBOSE,
            ),
            "setval": "match large-community {{ entries.match.large_community.community_list }}{{ (' exact-match') if mat is defined }}",
            "compval": "entries.match.large_community",
            "result": {
                "entries": [
                    {
                        "match": {
                            "large_community": {
                                "community_list": "{{ list }}",
                                "exact_match": "{{ True if mat is defined }}",
                            },
                        },
                    },
                ],
            },
        },
        {
            "name": "match.isis",
            "getval": re.compile(
                r"""
                \s*match\sisis\slevel
                \s+(?P<level>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "match isis level {{ entries.match.isis_level }}",
            "compval": "entries.match.isis_level",
            "result": {
                "entries": [
                    {
                        "match": {
                            "isis_level": "{{ level }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "match.local_pref",
            "getval": re.compile(
                r"""
                \s*match\slocal-preference
                \s+(?P<as>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "match local-preference {{ entries.match.local_preference }}",
            "compval": "entries.match.local_preference",
            "result": {
                "entries": [
                    {
                        "match": {
                            "local_preference": "{{ as }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "match.metric",
            "getval": re.compile(
                r"""
                \s*match\smetric
                \s+(?P<val>\d+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_route_maps_match_metric,
            "compval": "entries.match.metric",
            "result": {
                "entries": [
                    {
                        "match": {
                            "metric": "{{ val }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "match.metric_type",
            "getval": re.compile(
                r"""
                \s*match\smetric-type
                \s+(?P<type>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "match metric-type {{ entries.match.metric_type }}",
            "compval": "entries.match.local_preference",
            "result": {
                "entries": [
                    {
                        "match": {
                            "metric_type": "{{ type }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "match.route_type",
            "getval": re.compile(
                r"""
                \s*match\sroute-type
                \s+(?P<type>\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": "match route-type {{ entries.match.route_type }}",
            "compval": "entries.match.route_type",
            "result": {
                "entries": [
                    {
                        "match": {
                            "route_type": "{{ type }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "match.routerid",
            "getval": re.compile(
                r"""
                \s*match\srouter-id\sprefix-list
                \s+(?P<id>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "match router-id prefix-list {{ entries.match.router_id }}",
            "compval": "entries.match.router_id",
            "result": {
                "entries": [
                    {
                        "match": {
                            "router_id": "{{ id }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "match.source_protocol",
            "getval": re.compile(
                r"""
                \s*match\ssource-protocol
                \s+(?P<proto>\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": "match source-protocol {{ entries.match.source_protocol }}",
            "compval": "entries.match.source_protocol",
            "result": {
                "entries": [
                    {
                        "match": {
                            "source_protocol": "{{ proto }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "match.tag",
            "getval": re.compile(
                r"""
                \s*match\stag
                \s+(?P<val>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "match tag {{ entries.match.tag }}",
            "compval": "entries.match.tag",
            "result": {
                "entries": [
                    {
                        "match": {
                            "tag": "{{ val }}",
                        },
                    },
                ],
            },
        },
    ]
    # fmt: on



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The keep_keys plugin code
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from ansible.errors import AnsibleFilterError


def _raise_error(msg):
    """Raise an error message, prepend with filter name
    :param msg: The message
    :type msg: str
    :raises: AnsibleError
    """
    error = "Error when using plugin 'keep_keys': {msg}".format(msg=msg)
    raise AnsibleFilterError(error)


def keep_keys_from_dict_n_list(data, target, matching_parameter):
    if isinstance(data, list):
        list_data = [keep_keys_from_dict_n_list(each, target, matching_parameter) for each in data]
        return list_data
    if isinstance(data, dict):
        keep = {}
        for k, val in data.items():
            for key in target:
                match = None
                if not isinstance(val, (list, dict)):
                    if matching_parameter == "regex":
                        match = re.match(key, k)
                        if match:
                            keep[k] = val
                    elif matching_parameter == "starts_with":
                        if k.startswith(key):
                            keep[k], match = val, True
                    elif matching_parameter == "ends_with":
                        if k.endswith(key):
                            keep[k], match = val, True
                    else:
                        if k == key:
                            keep[k], match = val, True
                else:
                    list_data = keep_keys_from_dict_n_list(val, target, matching_parameter)
                    if isinstance(list_data, list) and not match:
                        list_data = [r for r in list_data if r not in ([], {})]
                        if all(isinstance(s, str) for s in list_data):
                            continue
                    if list_data in ([], {}):
                        continue
                    keep[k] = list_data
        return keep
    return data


def keep_keys(data, target, matching_parameter="equality"):
    """keep selective keys recursively from a given data"
    :param data: The data passed in (data|keep_keys(...))
    :type data: raw
    :param target: List of keys on with operation is to be performed
    :type data: list
    :type elements: string
    :param matching_parameter: matching type of the target keys with data keys
    :type data: str
    """
    if not isinstance(data, (list, dict)):
        _raise_error("Input is not valid for keep operation")
    data = keep_keys_from_dict_n_list(data, target, matching_parameter)
    return data



# -*- coding: utf-8 -*-

# Copyright (c) 2022, Håkon Lerring
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import copy
import json

from ansible.module_utils.six.moves.urllib import error as urllib_error
from ansible.module_utils.six.moves.urllib.parse import urlencode
from ansible.module_utils.urls import open_url


def get_consul_url(configuration):
    return "%s://%s:%s/v1" % (
        configuration.scheme,
        configuration.host,
        configuration.port,
    )


def get_auth_headers(configuration):
    if configuration.token is None:
        return {}
    else:
        return {"X-Consul-Token": configuration.token}


class RequestError(Exception):
    def __init__(self, status, response_data=None):
        self.status = status
        self.response_data = response_data

    def __str__(self):
        if self.response_data is None:
            # self.status is already the message (backwards compat)
            return self.status
        return "HTTP %d: %s" % (self.status, self.response_data)


def handle_consul_response_error(response):
    if 400 <= response.status_code < 600:
        raise RequestError("%d %s" % (response.status_code, response.content))


AUTH_ARGUMENTS_SPEC = dict(
    host=dict(default="localhost"),
    port=dict(type="int", default=8500),
    scheme=dict(default="http"),
    validate_certs=dict(type="bool", default=True),
    token=dict(no_log=True),
    ca_path=dict(),
)


def camel_case_key(key):
    parts = []
    for part in key.split("_"):
        if part in {"id", "ttl", "jwks", "jwt", "oidc", "iam", "sts"}:
            parts.append(part.upper())
        else:
            parts.append(part.capitalize())
    return "".join(parts)


STATE_PARAMETER = "state"
STATE_PRESENT = "present"
STATE_ABSENT = "absent"

OPERATION_READ = "read"
OPERATION_CREATE = "create"
OPERATION_UPDATE = "update"
OPERATION_DELETE = "remove"


def _normalize_params(params, arg_spec):
    final_params = {}
    for k, v in params.items():
        if k not in arg_spec:  # Alias
            continue
        spec = arg_spec[k]
        if (
            spec.get("type") == "list"
            and spec.get("elements") == "dict"
            and spec.get("options")
            and v
        ):
            v = [_normalize_params(d, spec["options"]) for d in v]
        elif spec.get("type") == "dict" and spec.get("options") and v:
            v = _normalize_params(v, spec["options"])
        final_params[k] = v
    return final_params


class _ConsulModule:
    """Base class for Consul modules.

    This class is considered private, till the API is fully fleshed out.
    As such backwards incompatible changes can occur even in bugfix releases.
    """

    api_endpoint = None  # type: str
    unique_identifier = None  # type: str
    result_key = None  # type: str
    create_only_fields = set()
    params = {}

    def __init__(self, module):
        self._module = module
        self.params = _normalize_params(module.params, module.argument_spec)
        self.api_params = {
            k: camel_case_key(k)
            for k in self.params
            if k not in STATE_PARAMETER and k not in AUTH_ARGUMENTS_SPEC
        }

    def execute(self):
        obj = self.read_object()

        changed = False
        diff = {}
        if self.params[STATE_PARAMETER] == STATE_PRESENT:
            obj_from_module = self.module_to_obj(obj is not None)
            if obj is None:
                operation = OPERATION_CREATE
                new_obj = self.create_object(obj_from_module)
                diff = {"before": {}, "after": new_obj}
                changed = True
            else:
                operation = OPERATION_UPDATE
                if self._needs_update(obj, obj_from_module):
                    new_obj = self.update_object(obj, obj_from_module)
                    diff = {"before": obj, "after": new_obj}
                    changed = True
                else:
                    new_obj = obj
        elif self.params[STATE_PARAMETER] == STATE_ABSENT:
            operation = OPERATION_DELETE
            if obj is not None:
                self.delete_object(obj)
                changed = True
                diff = {"before": obj, "after": {}}
            else:
                diff = {"before": {}, "after": {}}
            new_obj = None
        else:
            raise RuntimeError("Unknown state supplied.")

        result = {"changed": changed}
        if changed:
            result["operation"] = operation
            if self._module._diff:
                result["diff"] = diff
        if self.result_key:
            result[self.result_key] = new_obj
        self._module.exit_json(**result)

    def module_to_obj(self, is_update):
        obj = {}
        for k, v in self.params.items():
            result = self.map_param(k, v, is_update)
            if result:
                obj[result[0]] = result[1]
        return obj

    def map_param(self, k, v, is_update):
        def helper(item):
            return {camel_case_key(k): v for k, v in item.items()}

        def needs_camel_case(k):
            spec = self._module.argument_spec[k]
            return (
                spec.get("type") == "list"
                and spec.get("elements") == "dict"
                and spec.get("options")
            ) or (spec.get("type") == "dict" and spec.get("options"))

        if k in self.api_params and v is not None:
            if isinstance(v, dict) and needs_camel_case(k):
                v = helper(v)
            elif isinstance(v, (list, tuple)) and needs_camel_case(k):
                v = [helper(i) for i in v]
            if is_update and k in self.create_only_fields:
                return
            return camel_case_key(k), v

    def _needs_update(self, api_obj, module_obj):
        api_obj = copy.deepcopy(api_obj)
        module_obj = copy.deepcopy(module_obj)
        return self.needs_update(api_obj, module_obj)

    def needs_update(self, api_obj, module_obj):
        for k, v in module_obj.items():
            if k not in api_obj:
                return True
            if api_obj[k] != v:
                return True
        return False

    def prepare_object(self, existing, obj):
        operational_attributes = {"CreateIndex", "CreateTime", "Hash", "ModifyIndex"}
        existing = {
            k: v for k, v in existing.items() if k not in operational_attributes
        }
        for k, v in obj.items():
            existing[k] = v
        return existing

    def endpoint_url(self, operation, identifier=None):
        if operation == OPERATION_CREATE:
            return self.api_endpoint
        elif identifier:
            return "/".join([self.api_endpoint, identifier])
        raise RuntimeError("invalid arguments passed")

    def read_object(self):
        url = self.endpoint_url(OPERATION_READ, self.params.get(self.unique_identifier))
        try:
            return self.get(url)
        except RequestError as e:
            if e.status == 404:
                return
            elif e.status == 403 and b"ACL not found" in e.response_data:
                return
            raise

    def create_object(self, obj):
        if self._module.check_mode:
            return obj
        else:
            return self.put(self.api_endpoint, data=self.prepare_object({}, obj))

    def update_object(self, existing, obj):
        url = self.endpoint_url(
            OPERATION_UPDATE, existing.get(camel_case_key(self.unique_identifier))
        )
        merged_object = self.prepare_object(existing, obj)
        if self._module.check_mode:
            return merged_object
        else:
            return self.put(url, data=merged_object)

    def delete_object(self, obj):
        if self._module.check_mode:
            return {}
        else:
            url = self.endpoint_url(
                OPERATION_DELETE, obj.get(camel_case_key(self.unique_identifier))
            )
            return self.delete(url)

    def _request(self, method, url_parts, data=None, params=None):
        module_params = self.params

        if not isinstance(url_parts, (tuple, list)):
            url_parts = [url_parts]
        if params:
            # Remove values that are None
            params = {k: v for k, v in params.items() if v is not None}

        ca_path = module_params.get("ca_path")
        base_url = "%s://%s:%s/v1" % (
            module_params["scheme"],
            module_params["host"],
            module_params["port"],
        )
        url = "/".join([base_url] + list(url_parts))

        headers = {}
        token = self.params.get("token")
        if token:
            headers["X-Consul-Token"] = token

        try:
            if data is not None:
                data = json.dumps(data)
                headers["Content-Type"] = "application/json"
            if params:
                url = "%s?%s" % (url, urlencode(params))
            response = open_url(
                url,
                method=method,
                data=data,
                headers=headers,
                validate_certs=module_params["validate_certs"],
                ca_path=ca_path,
            )
            response_data = response.read()
            status = (
                response.status if hasattr(response, "status") else response.getcode()
            )

        except urllib_error.URLError as e:
            if isinstance(e, urllib_error.HTTPError):
                status = e.code
                response_data = e.fp.read()
            else:
                self._module.fail_json(
                    msg="Could not connect to consul agent at %s:%s, error was %s"
                    % (module_params["host"], module_params["port"], str(e))
                )
                raise

        if 400 <= status < 600:
            raise RequestError(status, response_data)

        return json.loads(response_data)

    def get(self, url_parts, **kwargs):
        return self._request("GET", url_parts, **kwargs)

    def put(self, url_parts, **kwargs):
        return self._request("PUT", url_parts, **kwargs)

    def delete(self, url_parts, **kwargs):
        return self._request("DELETE", url_parts, **kwargs)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_objects_facts
short_description: Get objects objects facts on Checkpoint over Web Services API
description:
  - Get objects facts on Checkpoint devices.
  - All operations are performed over Web Services API.
  - This module handles both operations, get a specific object and get several objects,
    For getting a specific object use the parameter 'name'.
version_added: "3.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  uid:
    description:
    - Object unique identifier.
    type: str
  uids:
    description:
      - List of UIDs of the objects to retrieve.
    type: list
    elements: str
  filter:
    description:
      - Search expression to filter objects by. The provided text should be exactly the same as it would be given in Smart Console. The logical
        operators in the expression ('AND', 'OR') should be provided in capital letters. By default, the search involves both a textual search and a IP
        search. To use IP search only, set the "ip-only" parameter to true.
    type: str
  ip_only:
    description:
      - If using "filter", use this field to search objects by their IP address only, without involving the textual search.<br><br>IP search use
        cases<br>&nbsp;&nbsp;&nbsp;&nbsp; <ul><li>Full IPv4 address matches for,<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; - Hosts, Check Point
        Hosts and Gateways with exact IPv4 match or with interfaces which subnet contains the search
        address<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; - IPv4 Networks and IPv4 Address Ranges that contain the search address</li>
        <br>&nbsp;&nbsp;&nbsp;&nbsp; <li>Partial IPv4 address matches for,<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; - Hosts, Networks, Check Point
        Hosts and Gateways with IPv4 address that starts from the search address<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; - Hosts, Check Point
        Hosts and Gateways with interfaces which subnet address starts from the search address<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; - IPv4
        Address Ranges with first address or last address that starts from the search address<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; - IPv4
        Networks and IPv4 Address Ranges that contain the network derived from the search address supplemented with missing octets (all
        zeroes)<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; - Hosts, Check Point Hosts and Gateways with interfaces which subnet contains the network
        derived from the search address supplemented with missing octets (all zeroes)</li><br>&nbsp;&nbsp;&nbsp;&nbsp; <li>IPv6
        address,<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; - Not supported</li></ul><br><br> * Check Point Host is a server of type Network Policy
        Management, Logging & Status, SmartEvent, etc.<br> * When one IP address is checked to start from another (partial) IP address - only full octets are
        considered <br> * Check Examples part for IP search examples.
    type: bool
  limit:
    description:
      - The maximal number of returned results.
        This parameter is relevant only for getting a specific object.
    type: int
  offset:
    description:
      - Number of the results to initially skip.
        This parameter is relevant only for getting a specific object.
    type: int
  order:
    description:
      - Sorts the results by search criteria. Automatically sorts the results by Name, in the ascending order.
        This parameter is relevant only for getting a specific object.
    type: list
    elements: dict
    suboptions:
      ASC:
        description:
          - Sorts results by the given field in ascending order.
        type: str
        choices: ['name']
      DESC:
        description:
          - Sorts results by the given field in descending order.
        type: str
        choices: ['name']
  type:
    description:
      - The objects' type, e.g., host, service-tcp, network, address-range...
    type: str
  dereference_group_members:
    description:
      - Indicates whether to dereference "members" field by details level for every object in reply.
    type: bool
  show_membership:
    description:
      - Indicates whether to calculate and show "groups" field for every object in reply.
    type: bool
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
extends_documentation_fragment: check_point.mgmt.checkpoint_facts
"""

EXAMPLES = """
- name: show-objects
  cp_mgmt_objects_facts:
    limit: 50
    offset: 0
    order:
    - ASC: name
    type: group

- name: show-object
  cp_mgmt_objects_facts:
    uid: ef82887c-d08f-49a3-a18f-a376be633848
"""

RETURN = """
ansible_facts:
  description: The checkpoint object facts.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_facts,
    api_call_facts,
)


def main():
    argument_spec = dict(
        uid=dict(type="str"),
        uids=dict(type="list", elements="str"),
        filter=dict(type="str"),
        ip_only=dict(type="bool"),
        limit=dict(type="int"),
        offset=dict(type="int"),
        order=dict(
            type="list",
            elements="dict",
            options=dict(
                ASC=dict(type="str", choices=["name"]),
                DESC=dict(type="str", choices=["name"]),
            ),
        ),
        type=dict(type="str"),
        dereference_group_members=dict(type="bool"),
        show_membership=dict(type="bool"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        domains_to_process=dict(type="list", elements="str"),
    )
    argument_spec.update(checkpoint_argument_spec_for_facts)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )

    api_call_object = "object"
    api_call_object_plural_version = "objects"

    result = api_call_facts(
        module, api_call_object, api_call_object_plural_version
    )
    module.exit_json(ansible_facts=result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright (c), Google Inc, 2017
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import re
import time
import traceback

THIRD_LIBRARIES_IMP_ERR = None
try:
    from keystoneauth1.adapter import Adapter
    from keystoneauth1.identity import v3
    from keystoneauth1 import session
    HAS_THIRD_LIBRARIES = True
except ImportError:
    THIRD_LIBRARIES_IMP_ERR = traceback.format_exc()
    HAS_THIRD_LIBRARIES = False

from ansible.module_utils.basic import (AnsibleModule, env_fallback,
                                        missing_required_lib)
from ansible.module_utils.common.text.converters import to_text


class HwcModuleException(Exception):
    def __init__(self, message):
        super(HwcModuleException, self).__init__()

        self._message = message

    def __str__(self):
        return "[HwcClientException] message=%s" % self._message


class HwcClientException(Exception):
    def __init__(self, code, message):
        super(HwcClientException, self).__init__()

        self._code = code
        self._message = message

    def __str__(self):
        msg = " code=%s," % str(self._code) if self._code != 0 else ""
        return "[HwcClientException]%s message=%s" % (
            msg, self._message)


class HwcClientException404(HwcClientException):
    def __init__(self, message):
        super(HwcClientException404, self).__init__(404, message)

    def __str__(self):
        return "[HwcClientException404] message=%s" % self._message


def session_method_wrapper(f):
    def _wrap(self, url, *args, **kwargs):
        try:
            url = self.endpoint + url
            r = f(self, url, *args, **kwargs)
        except Exception as ex:
            raise HwcClientException(
                0, "Sending request failed, error=%s" % ex)

        result = None
        if r.content:
            try:
                result = r.json()
            except Exception as ex:
                raise HwcClientException(
                    0, "Parsing response to json failed, error: %s" % ex)

        code = r.status_code
        if code not in [200, 201, 202, 203, 204, 205, 206, 207, 208, 226]:
            msg = ""
            for i in ['message', 'error.message']:
                try:
                    msg = navigate_value(result, i)
                    break
                except Exception:
                    pass
            else:
                msg = str(result)

            if code == 404:
                raise HwcClientException404(msg)

            raise HwcClientException(code, msg)

        return result

    return _wrap


class _ServiceClient(object):
    def __init__(self, client, endpoint, product):
        self._client = client
        self._endpoint = endpoint
        self._default_header = {
            'User-Agent': "Huawei-Ansible-MM-%s" % product,
            'Accept': 'application/json',
        }

    @property
    def endpoint(self):
        return self._endpoint

    @endpoint.setter
    def endpoint(self, e):
        self._endpoint = e

    @session_method_wrapper
    def get(self, url, body=None, header=None, timeout=None):
        return self._client.get(url, json=body, timeout=timeout,
                                headers=self._header(header))

    @session_method_wrapper
    def post(self, url, body=None, header=None, timeout=None):
        return self._client.post(url, json=body, timeout=timeout,
                                 headers=self._header(header))

    @session_method_wrapper
    def delete(self, url, body=None, header=None, timeout=None):
        return self._client.delete(url, json=body, timeout=timeout,
                                   headers=self._header(header))

    @session_method_wrapper
    def put(self, url, body=None, header=None, timeout=None):
        return self._client.put(url, json=body, timeout=timeout,
                                headers=self._header(header))

    def _header(self, header):
        if header and isinstance(header, dict):
            for k, v in self._default_header.items():
                if k not in header:
                    header[k] = v
        else:
            header = self._default_header

        return header


class Config(object):
    def __init__(self, module, product):
        self._project_client = None
        self._domain_client = None
        self._module = module
        self._product = product
        self._endpoints = {}

        self._validate()
        self._gen_provider_client()

    @property
    def module(self):
        return self._module

    def client(self, region, service_type, service_level):
        c = self._project_client
        if service_level == "domain":
            c = self._domain_client

        e = self._get_service_endpoint(c, service_type, region)

        return _ServiceClient(c, e, self._product)

    def _gen_provider_client(self):
        m = self._module
        p = {
            "auth_url": m.params['identity_endpoint'],
            "password": m.params['password'],
            "username": m.params['user'],
            "project_name": m.params['project'],
            "user_domain_name": m.params['domain'],
            "reauthenticate": True
        }

        self._project_client = Adapter(
            session.Session(auth=v3.Password(**p)),
            raise_exc=False)

        p.pop("project_name")
        self._domain_client = Adapter(
            session.Session(auth=v3.Password(**p)),
            raise_exc=False)

    def _get_service_endpoint(self, client, service_type, region):
        k = "%s.%s" % (service_type, region if region else "")

        if k in self._endpoints:
            return self._endpoints.get(k)

        url = None
        try:
            url = client.get_endpoint(service_type=service_type,
                                      region_name=region, interface="public")
        except Exception as ex:
            raise HwcClientException(
                0, "Getting endpoint failed, error=%s" % ex)

        if url == "":
            raise HwcClientException(
                0, "Cannot find the endpoint for %s" % service_type)

        if url[-1] != "/":
            url += "/"

        self._endpoints[k] = url
        return url

    def _validate(self):
        if not HAS_THIRD_LIBRARIES:
            self.module.fail_json(
                msg=missing_required_lib('keystoneauth1'),
                exception=THIRD_LIBRARIES_IMP_ERR)


class HwcModule(AnsibleModule):
    def __init__(self, *args, **kwargs):
        arg_spec = kwargs.setdefault('argument_spec', {})

        arg_spec.update(
            dict(
                identity_endpoint=dict(
                    required=True, type='str',
                    fallback=(env_fallback, ['ANSIBLE_HWC_IDENTITY_ENDPOINT']),
                ),
                user=dict(
                    required=True, type='str',
                    fallback=(env_fallback, ['ANSIBLE_HWC_USER']),
                ),
                password=dict(
                    required=True, type='str', no_log=True,
                    fallback=(env_fallback, ['ANSIBLE_HWC_PASSWORD']),
                ),
                domain=dict(
                    required=True, type='str',
                    fallback=(env_fallback, ['ANSIBLE_HWC_DOMAIN']),
                ),
                project=dict(
                    required=True, type='str',
                    fallback=(env_fallback, ['ANSIBLE_HWC_PROJECT']),
                ),
                region=dict(
                    type='str',
                    fallback=(env_fallback, ['ANSIBLE_HWC_REGION']),
                ),
                id=dict(type='str')
            )
        )

        super(HwcModule, self).__init__(*args, **kwargs)


class _DictComparison(object):
    ''' This class takes in two dictionaries `a` and `b`.
        These are dictionaries of arbitrary depth, but made up of standard
        Python types only.
        This differ will compare all values in `a` to those in `b`.
        If value in `a` is None, always returns True, indicating
        this value is no need to compare.
        Note: On all lists, order does matter.
    '''

    def __init__(self, request):
        self.request = request

    def __eq__(self, other):
        return self._compare_dicts(self.request, other.request)

    def __ne__(self, other):
        return not self.__eq__(other)

    def _compare_dicts(self, dict1, dict2):
        if dict1 is None:
            return True

        if set(dict1.keys()) != set(dict2.keys()):
            return False

        for k in dict1:
            if not self._compare_value(dict1.get(k), dict2.get(k)):
                return False

        return True

    def _compare_lists(self, list1, list2):
        """Takes in two lists and compares them."""
        if list1 is None:
            return True

        if len(list1) != len(list2):
            return False

        for i in range(len(list1)):
            if not self._compare_value(list1[i], list2[i]):
                return False

        return True

    def _compare_value(self, value1, value2):
        """
        return: True: value1 is same as value2, otherwise False.
        """
        if value1 is None:
            return True

        if not (value1 and value2):
            return (not value1) and (not value2)

        # Can assume non-None types at this point.
        if isinstance(value1, list) and isinstance(value2, list):
            return self._compare_lists(value1, value2)

        elif isinstance(value1, dict) and isinstance(value2, dict):
            return self._compare_dicts(value1, value2)

        # Always use to_text values to avoid unicode issues.
        return (to_text(value1, errors='surrogate_or_strict') == to_text(
            value2, errors='surrogate_or_strict'))


def wait_to_finish(target, pending, refresh, timeout, min_interval=1, delay=3):
    is_last_time = False
    not_found_times = 0
    wait = 0

    time.sleep(delay)

    end = time.time() + timeout
    while not is_last_time:
        if time.time() > end:
            is_last_time = True

        obj, status = refresh()

        if obj is None:
            not_found_times += 1

            if not_found_times > 10:
                raise HwcModuleException(
                    "not found the object for %d times" % not_found_times)
        else:
            not_found_times = 0

            if status in target:
                return obj

            if pending and status not in pending:
                raise HwcModuleException(
                    "unexpected status(%s) occurred" % status)

        if not is_last_time:
            wait *= 2
            if wait < min_interval:
                wait = min_interval
            elif wait > 10:
                wait = 10

            time.sleep(wait)

    raise HwcModuleException("async wait timeout after %d seconds" % timeout)


def navigate_value(data, index, array_index=None):
    if array_index and (not isinstance(array_index, dict)):
        raise HwcModuleException("array_index must be dict")

    d = data
    for n in range(len(index)):
        if d is None:
            return None

        if not isinstance(d, dict):
            raise HwcModuleException(
                "can't navigate value from a non-dict object")

        i = index[n]
        if i not in d:
            raise HwcModuleException(
                "navigate value failed: key(%s) is not exist in dict" % i)
        d = d[i]

        if not array_index:
            continue

        k = ".".join(index[: (n + 1)])
        if k not in array_index:
            continue

        if d is None:
            return None

        if not isinstance(d, list):
            raise HwcModuleException(
                "can't navigate value from a non-list object")

        j = array_index.get(k)
        if j >= len(d):
            raise HwcModuleException(
                "navigate value failed: the index is out of list")
        d = d[j]

    return d


def build_path(module, path, kv=None):
    if kv is None:
        kv = dict()

    v = {}
    for p in re.findall(r"{[^/]*}", path):
        n = p[1:][:-1]

        if n in kv:
            v[n] = str(kv[n])

        else:
            if n in module.params:
                v[n] = str(module.params.get(n))
            else:
                v[n] = ""

    return path.format(**v)


def get_region(module):
    if module.params['region']:
        return module.params['region']

    return module.params['project'].split("_")[0]


def is_empty_value(v):
    return (not v)


def are_different_dicts(dict1, dict2):
    return _DictComparison(dict1) != _DictComparison(dict2)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (@dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_user
short_description: Manage users
description:
- Manage users on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
options:
  user:
    description:
    - The name of the user.
    type: str
    aliases: [ name ]
  user_password:
    description:
    - The password of the user.
    type: str
  first_name:
    description:
    - The first name of the user.
    - This parameter is required when creating new users.
    type: str
  last_name:
    description:
    - The last name of the user.
    - This parameter is required when creating new users.
    type: str
  email:
    description:
    - The email address of the user.
    - This parameter is required when creating new users.
    type: str
  phone:
    description:
    - The phone number of the user.
    - This parameter is required when creating new users.
    type: str
  account_status:
    description:
    - The status of the user account.
    type: str
    choices: [ active, inactive ]
  domain:
    description:
    - The domain this user belongs to.
    - When creating new users, this defaults to C(Local).
    type: str
  roles:
    description:
    - The roles for this user and their access types (read or write).
    - Access type defaults to C(write).
    type: list
    elements: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
notes:
- A default installation of ACI Multi-Site ships with admin password 'we1come!' which requires a password change on first login.
  See the examples of how to change the 'admin' password using Ansible.
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Update initial admin password
  cisco.mso.mso_user:
    host: mso_host
    username: admin
    password: initialPassword
    validate_certs: false
    user: admin
    user_password: newPassword
    state: present
  delegate_to: localhost

- name: Add a new user
  cisco.mso.mso_user:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    validate_certs: false
    user: dag
    user_password: userPassword
    first_name: Dag
    last_name: Wieers
    email: dag@wieers.com
    phone: +32 478 436 299
    roles:
    - name: siteManager
      access_type: write
    - name: schemaManager
      access_type: read
    state: present
  delegate_to: localhost

- name: Add a new user
  cisco.mso.mso_user:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    validate_certs: false
    user: dag
    first_name: Dag
    last_name: Wieers
    email: dag@wieers.com
    phone: +32 478 436 299
    roles:
    - powerUser
  delegate_to: localhost

- name: Remove a user
  cisco.mso.mso_user:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    validate_certs: false
    user: dag
    state: absent
  delegate_to: localhost

- name: Query a user
  cisco.mso.mso_user:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    validate_certs: false
    user: dag
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all users
  cisco.mso.mso_user:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    validate_certs: false
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r""" # """

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, issubset


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        user=dict(type="str", aliases=["name"]),
        user_password=dict(type="str", no_log=True),
        first_name=dict(type="str"),
        last_name=dict(type="str"),
        email=dict(type="str"),
        phone=dict(type="str"),
        # TODO: What possible options do we have ?
        account_status=dict(type="str", choices=["active", "inactive"]),
        domain=dict(type="str"),
        roles=dict(type="list", elements="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["user"]],
            ["state", "present", ["user"]],
        ],
    )

    user_name = module.params.get("user")
    user_password = module.params.get("user_password")
    first_name = module.params.get("first_name")
    last_name = module.params.get("last_name")
    email = module.params.get("email")
    phone = module.params.get("phone")
    account_status = module.params.get("account_status")
    state = module.params.get("state")

    mso = MSOModule(module)

    roles = mso.lookup_roles(module.params.get("roles"))
    domain = mso.lookup_domain(module.params.get("domain"))

    user_id = None
    path = "users"

    # Query for existing object(s)
    if user_name:
        if mso.module._socket_path and mso.connection.get_platform() == "cisco.nd":
            mso.existing = mso.get_obj(path, loginID=user_name, api_version="v2")
            if mso.existing:
                mso.existing["id"] = mso.existing.get("userID")
                mso.existing["username"] = mso.existing.get("loginID")
        else:
            mso.existing = mso.get_obj(path, username=user_name)
        if mso.existing:
            user_id = mso.existing.get("id")
            # If we found an existing object, continue with it
            path = "users/{id}".format(id=user_id)
    else:
        mso.existing = mso.query_objs(path)

    if state == "query":
        pass

    elif state == "absent":
        mso.previous = mso.existing
        if mso.existing:
            if module.check_mode:
                mso.existing = {}
            else:
                mso.existing = mso.request(path, method="DELETE")

    elif state == "present":
        mso.previous = mso.existing

        payload = dict(
            id=user_id,
            username=user_name,
            firstName=first_name,
            lastName=last_name,
            emailAddress=email,
            phoneNumber=phone,
            accountStatus=account_status,
            domainId=domain,
            roles=roles,
            # active=True,
            # remote=True,
        )

        if user_password is not None:
            payload.update(password=user_password)

        mso.sanitize(payload, collate=True)

        if mso.sent.get("accountStatus") is None:
            mso.sent["accountStatus"] = "active"

        if mso.existing:
            if not issubset(mso.sent, mso.existing):
                # NOTE: Since MSO always returns '******' as password, we need to assume a change
                if "password" in mso.proposed:
                    mso.module.warn("A password change is assumed, as the MSO REST API does not return passwords we do not know.")
                    mso.result["changed"] = True

                if module.check_mode:
                    mso.existing = mso.proposed
                else:
                    mso.existing = mso.request(path, method="PUT", data=mso.sent)

        else:
            if user_password is None:
                mso.fail_json("The user {0} does not exist. The 'user_password' attribute is required to create a new user.".format(user_name))
            if module.check_mode:
                mso.existing = mso.proposed
            else:
                mso.existing = mso.request(path, method="POST", data=mso.sent)

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_appliance_security_intrusion
short_description: Resource module for networks _appliance _security _intrusion
description:
- Manage operation update of the resource networks _appliance _security _intrusion.
- Set the supported intrusion settings for an MX network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  idsRulesets:
    description: Set the detection ruleset 'connectivity'/'balanced'/'security' (optional
      - omitting will leave current config unchanged). Default value is 'balanced' if
      none currently saved.
    type: str
  mode:
    description: Set mode to 'disabled'/'detection'/'prevention' (optional - omitting
      will leave current config unchanged).
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  protectedNetworks:
    description: Set the included/excluded networks from the intrusion engine (optional
      - omitting will leave current config unchanged). This is available only in 'passthrough'
      mode.
    suboptions:
      excludedCidr:
        description: List of IP addresses or subnets being excluded from protection
          (required if 'useDefault' is false).
        elements: str
        type: list
      includedCidr:
        description: List of IP addresses or subnets being protected (required if 'useDefault'
          is false).
        elements: str
        type: list
      useDefault:
        description: True/false whether to use special IPv4 addresses https //tools.ietf.org/html/rfc5735
          (required). Default value is true if none currently saved.
        type: bool
    type: dict
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for appliance updateNetworkApplianceSecurityIntrusion
  description: Complete reference of the updateNetworkApplianceSecurityIntrusion API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-security-intrusion
notes:
  - SDK Method used are
    appliance.Appliance.update_network_appliance_security_intrusion,

  - Paths used are
    put /networks/{networkId}/appliance/security/intrusion,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_appliance_security_intrusion:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    idsRulesets: balanced
    mode: prevention
    networkId: string
    protectedNetworks:
      excludedCidr:
      - 10.0.0.0/8
      - 127.0.0.0/8
      includedCidr:
      - 10.0.0.0/8
      - 127.0.0.0/8
      - 169.254.0.0/16
      - 172.16.0.0/12
      useDefault: false

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sts_assume_role
version_added: 1.0.0
version_added_collection: community.aws
short_description: Assume a role using AWS Security Token Service and obtain temporary credentials
description:
  - Assume a role using AWS Security Token Service and obtain temporary credentials.
author:
  - Boris Ekelchik (@bekelchik)
  - Marek Piatek (@piontas)
options:
  role_arn:
    description:
      - The Amazon Resource Name (ARN) of the role that the caller is
        assuming U(https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html#Identifiers_ARNs).
    required: true
    type: str
  role_session_name:
    description:
      - Name of the role's session - will be used by CloudTrail.
    required: true
    type: str
  policy:
    description:
      - Supplemental policy to use in addition to assumed role's policies.
    type: str
  duration_seconds:
    description:
      - The duration, in seconds, of the role session. The value can range from 900 seconds (15 minutes) to 43200 seconds (12 hours).
      - The max depends on the IAM role's sessions duration setting.
      - By default, the value is set to 3600 seconds.
    type: int
  external_id:
    description:
      - A unique identifier that is used by third parties to assume a role in their customers' accounts.
    type: str
  mfa_serial_number:
    description:
      - The identification number of the MFA device that is associated with the user who is making the AssumeRole call.
    type: str
  mfa_token:
    description:
      - The value provided by the MFA device, if the trust policy of the role being assumed requires MFA.
    type: str
notes:
  - In order to use the assumed role in a following playbook task you must pass the I(access_key),
    I(secret_key) and I(session_token) parameters to modules that should use the assumed credentials.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

RETURN = r"""
sts_creds:
    description: The temporary security credentials, which include an access key ID, a secret access key, and a security (or session) token
    returned: always
    type: dict
    sample:
      access_key: XXXXXXXXXXXXXXXXXXXX
      expiration: '2017-11-11T11:11:11+00:00'
      secret_key: XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
      session_token: XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
sts_user:
    description: The Amazon Resource Name (ARN) and the assumed role ID
    returned: always
    type: dict
    sample:
      assumed_role_id: arn:aws:sts::123456789012:assumed-role/demo/Bob
      arn: ARO123EXAMPLE123:Bob
changed:
    description: True if obtaining the credentials succeeds
    type: bool
    returned: always
"""

EXAMPLES = r"""
# Assume an existing role (more details: https://docs.aws.amazon.com/STS/latest/APIReference/API_AssumeRole.html)
- amazon.aws.sts_assume_role:
    access_key: AKIA1EXAMPLE1EXAMPLE
    secret_key: 123456789abcdefghijklmnopqrstuvwxyzABCDE
    role_arn: "arn:aws:iam::123456789012:role/someRole"
    role_session_name: "someRoleSession"
  register: assumed_role

# Use the assumed role above to tag an instance in account 123456789012
- amazon.aws.ec2_tag:
    access_key: "{{ assumed_role.sts_creds.access_key }}"
    secret_key: "{{ assumed_role.sts_creds.secret_key }}"
    session_token: "{{ assumed_role.sts_creds.session_token }}"
    resource: i-xyzxyz01
    state: present
    tags:
      MyNewTag: value
"""

try:
    from botocore.exceptions import ClientError
    from botocore.exceptions import ParamValidationError
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule


def _parse_response(response):
    credentials = response.get("Credentials", {})
    user = response.get("AssumedRoleUser", {})

    sts_cred = {
        "access_key": credentials.get("AccessKeyId"),
        "secret_key": credentials.get("SecretAccessKey"),
        "session_token": credentials.get("SessionToken"),
        "expiration": credentials.get("Expiration"),
    }
    sts_user = camel_dict_to_snake_dict(user)
    return sts_cred, sts_user


def assume_role_policy(connection, module):
    params = {
        "RoleArn": module.params.get("role_arn"),
        "RoleSessionName": module.params.get("role_session_name"),
        "Policy": module.params.get("policy"),
        "DurationSeconds": module.params.get("duration_seconds"),
        "ExternalId": module.params.get("external_id"),
        "SerialNumber": module.params.get("mfa_serial_number"),
        "TokenCode": module.params.get("mfa_token"),
    }
    changed = False

    kwargs = dict((k, v) for k, v in params.items() if v is not None)

    try:
        response = connection.assume_role(**kwargs)
        changed = True
    except (ClientError, ParamValidationError) as e:
        module.fail_json_aws(e)

    sts_cred, sts_user = _parse_response(response)
    module.exit_json(changed=changed, sts_creds=sts_cred, sts_user=sts_user)


def main():
    argument_spec = dict(
        role_arn=dict(required=True),
        role_session_name=dict(required=True),
        duration_seconds=dict(required=False, default=None, type="int"),
        external_id=dict(required=False, default=None),
        policy=dict(required=False, default=None),
        mfa_serial_number=dict(required=False, default=None),
        mfa_token=dict(required=False, default=None, no_log=True),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec)

    connection = module.client("sts")

    assume_role_policy(connection, module)


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The pop_ace filter plugin
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
name: pop_ace
author: Sagar Paul (@KB-perByte)
version_added: "5.1.0"
short_description: Remove ace entries from a acl source of truth.
description:
    - This plugin removes specific keys from a provided acl data.
    - Using the parameters below - C(acls_data | ansible.netcommon.pop_ace(filter_options=filter_options, match_criteria=match_criteria))
notes:
  - The filter plugin has been tested with facts collected for acls resource module on
    Cisco IOSXE, IOSXR and NXOS.
options:
  data:
    description:
    - This option represents a list of dictionaries of acls facts.
    - For example C(acls_data | ansible.netcommon.pop_ace(filter_options=filter_options, match_criteria=match_criteria)),
      in this case C(acls_data) represents this option.
    type: raw
    required: True
  filter_options:
    description: Specify filtering options which drives the filter plugin.
    type: dict
    suboptions:
      remove:
        description:
          - Remove first removes one ace from each ACL entry on match
          - Remove all is more aggressive and removes more than one on match
        type: str
        choices: ['first', 'all']
        default: all
      failed_when:
        description:
          - On missing it fails when there is no match with the ACL data supplied
          - On never it would never fail
        type: str
        choices: ['missing', 'never']
        default: missing
      match_all:
        description: When true ensures ace removed only when it matches all match criteria
        type: bool
        default: False
  match_criteria:
    description: Specify the matching configuration of the ACEs to remove.
    type: dict
    required: True
    suboptions:
      afi:
        description: Specify afi to match
        type: str
        required: True
      acl_name:
        description: ACL name to match
        type: str
      source:
        description: Source address/ host/ any of the ACE to match
        type: str
      destination:
        description: Destination address/ host/ any of the ACE to natch
        type: str
      sequence:
        description: Sequence number of the ACE to match
        type: str
      protocol:
        description: Protocol name of the ACE to match
        type: str
      grant:
        description: Grant type permit or deny to match
        type: str
"""

EXAMPLES = r"""
##Playbook with filter plugin example
vars:
  filter_options:
    match_all: true
  match_criteria:
    afi: "ipv4"
    source: "192.0.2.0"
    destination: "192.0.3.0"
  acls_data:
    - acls:
        - aces:
            - destination:
                address: 192.0.3.0
                wildcard_bits: 0.0.0.255
              dscp: ef
              grant: deny
              protocol: icmp
              protocol_options:
                icmp:
                  traceroute: true
              sequence: 10
              source:
                address: 192.0.2.0
                wildcard_bits: 0.0.0.255
              ttl:
                eq: 10
            - destination:
                host: 198.51.110.0
                port_protocol:
                  eq: telnet
              grant: deny
              protocol: tcp
              protocol_options:
                tcp:
                  ack: true
              sequence: 20
              source:
                host: 198.51.100.0
          acl_type: extended
          name: "110"
        - aces:
            - destination:
                address: 198.51.101.0
                port_protocol:
                  eq: telnet
                wildcard_bits: 0.0.0.255
              grant: deny
              protocol: tcp
              protocol_options:
                tcp:
                  ack: true
              sequence: 10
              source:
                address: 198.51.100.0
                wildcard_bits: 0.0.0.255
              tos:
                service_value: 12
            - destination:
                address: 192.0.4.0
                port_protocol:
                  eq: www
                wildcard_bits: 0.0.0.255
              dscp: ef
              grant: deny
              protocol: tcp
              protocol_options:
                tcp:
                  ack: true
              sequence: 20
              source:
                address: 192.0.3.0
                wildcard_bits: 0.0.0.255
              ttl:
                lt: 20
          acl_type: extended
          name: "123"
        - aces:
            - grant: deny
              sequence: 10
              source:
                host: 192.168.1.200
            - grant: deny
              sequence: 20
              source:
                address: 192.168.2.0
                wildcard_bits: 0.0.0.255
          acl_type: standard
          name: std_acl
        - aces:
            - destination:
                address: 192.0.3.0
                port_protocol:
                  eq: www
                wildcard_bits: 0.0.0.255
              grant: deny
              option:
                traceroute: true
              protocol: tcp
              protocol_options:
                tcp:
                  fin: true
              sequence: 10
              source:
                address: 192.0.2.0
                wildcard_bits: 0.0.0.255
              ttl:
                eq: 10
          acl_type: extended
          name: test
      afi: ipv4
    - acls:
        - aces:
            - destination:
                any: true
                port_protocol:
                  eq: telnet
              dscp: af11
              grant: deny
              protocol: tcp
              protocol_options:
                tcp:
                  ack: true
              sequence: 10
              source:
                any: true
                port_protocol:
                  eq: www
          name: R1_TRAFFIC
      afi: ipv6

tasks:
  - name: Remove ace entries from a provided data
    ansible.builtin.debug:
      msg: "{{ acls_data | ansible.netcommon.pop_ace(filter_options=filter_options, match_criteria=match_criteria) }}"

##Output
# PLAY [Filter plugin example pop_ace] ******************************************************************************************************************

# TASK [Remove ace entries from a provided data] ***********************************************************************************************************
# ok: [xe_machine] =>
#   msg:
#     clean_acls:
#       acls:
#       - acls:
#         - aces:
#           - destination:
#               host: 198.51.110.0
#               port_protocol:
#                 eq: telnet
#             grant: deny
#             protocol: tcp
#             protocol_options:
#               tcp:
#                 ack: true
#             sequence: 20
#             source:
#               host: 198.51.100.0
#           name: '110'
#         - aces:
#           - destination:
#               address: 198.51.101.0
#               port_protocol:
#                 eq: telnet
#               wildcard_bits: 0.0.0.255
#             grant: deny
#             protocol: tcp
#             protocol_options:
#               tcp:
#                 ack: true
#             sequence: 10
#             source:
#               address: 198.51.100.0
#               wildcard_bits: 0.0.0.255
#             tos:
#               service_value: 12
#           - destination:
#               address: 192.0.4.0
#               port_protocol:
#                 eq: www
#               wildcard_bits: 0.0.0.255
#             dscp: ef
#             grant: deny
#             protocol: tcp
#             protocol_options:
#               tcp:
#                 ack: true
#             sequence: 20
#             source:
#               address: 192.0.3.0
#               wildcard_bits: 0.0.0.255
#             ttl:
#               lt: 20
#           name: '123'
#         - aces:
#           - grant: deny
#             sequence: 10
#             source:
#               host: 192.168.1.200
#           - grant: deny
#             sequence: 20
#             source:
#               address: 192.168.2.0
#               wildcard_bits: 0.0.0.255
#           name: std_acl
#         afi: ipv4
#       - acls:
#         - aces:
#           - destination:
#               any: true
#               port_protocol:
#                 eq: telnet
#             dscp: af11
#             grant: deny
#             protocol: tcp
#             protocol_options:
#               tcp:
#                 ack: true
#             sequence: 10
#             source:
#               any: true
#               port_protocol:
#                 eq: www
#           name: R1_TRAFFIC
#         afi: ipv6
#     removed_aces:
#       acls:
#       - acls:
#         - aces:
#           - destination:
#               address: 192.0.3.0
#               wildcard_bits: 0.0.0.255
#             dscp: ef
#             grant: deny
#             protocol: icmp
#             protocol_options:
#               icmp:
#                 traceroute: true
#             sequence: 10
#             source:
#               address: 192.0.2.0
#               wildcard_bits: 0.0.0.255
#             ttl:
#               eq: 10
#           name: '110'
#         - aces:
#           - destination:
#               address: 192.0.3.0
#               port_protocol:
#                 eq: www
#               wildcard_bits: 0.0.0.255
#             grant: deny
#             option:
#               traceroute: true
#             protocol: tcp
#             protocol_options:
#               tcp:
#                 fin: true
#             sequence: 10
#             source:
#               address: 192.0.2.0
#               wildcard_bits: 0.0.0.255
#             ttl:
#               eq: 10
#           name: test
#         afi: ipv4
#       - acls: []
#         afi: ipv6


##Playbook with workflow example
tasks:
  - name: Gather ACLs config from device existing ACLs config
    cisco.ios.ios_acls:
      state: gathered
    register: result_gathered

  - name: Setting host facts for pop_ace filter plugin
    ansible.builtin.set_fact:
      acls_facts: "{{ result_gathered.gathered }}"
      filter_options:
        match_all: true
      match_criteria:
        afi: "ipv4"
        source: "192.0.2.0"
        destination: "192.0.3.0"

  - name: Invoke pop_ace filter plugin
    ansible.builtin.set_fact:
      clean_acls: "{{ acls_facts | ansible.netcommon.pop_ace(filter_options=filter_options, match_criteria=match_criteria) }}"

  - name: Override ACLs config with device existing ACLs config
    cisco.ios.ios_acls:
      state: overridden
      config: "{{ clean_acls['clean_acls']['acls'] | from_yaml }}"


##Output

# PLAYBOOK: pop_ace_example.yml ***********************************************

# PLAY [Filter plugin example pop_ace] ****************************************

# TASK [Gather ACLs config with device existing ACLs config] *********************
# ok: [xe_machine] => changed=false
#   gathered:
#   - acls:
#     - aces:
#       - destination:
#           address: 192.0.3.0
#           wildcard_bits: 0.0.0.255
#         dscp: ef
#         grant: deny
#         protocol: icmp
#         protocol_options:
#           icmp:
#             traceroute: true
#         sequence: 10
#         source:
#           address: 192.0.2.0
#           wildcard_bits: 0.0.0.255
#         ttl:
#           eq: 10
#       - destination:
#           host: 198.51.110.0
#           port_protocol:
#             eq: telnet
#         grant: deny
#         protocol: tcp
#         protocol_options:
#           tcp:
#             ack: true
#         sequence: 20
#         source:
#           host: 198.51.100.0
#       acl_type: extended
#       name: '110'
#     - aces:
#       - destination:
#           address: 198.51.101.0
#           port_protocol:
#             eq: telnet
#           wildcard_bits: 0.0.0.255
#         grant: deny
#         protocol: tcp
#         protocol_options:
#           tcp:
#             ack: true
#         sequence: 10
#         source:
#           address: 198.51.100.0
#           wildcard_bits: 0.0.0.255
#         tos:
#           service_value: 12
#       - destination:
#           address: 192.0.4.0
#           port_protocol:
#             eq: www
#           wildcard_bits: 0.0.0.255
#         dscp: ef
#         grant: deny
#         protocol: tcp
#         protocol_options:
#           tcp:
#             ack: true
#         sequence: 20
#         source:
#           address: 192.0.3.0
#           wildcard_bits: 0.0.0.255
#         ttl:
#           lt: 20
#       acl_type: extended
#       name: '123'
#     - aces:
#       - grant: deny
#         sequence: 10
#         source:
#           host: 192.168.1.200
#       - grant: deny
#         sequence: 20
#         source:
#           address: 192.168.2.0
#           wildcard_bits: 0.0.0.255
#       acl_type: standard
#       name: std_acl
#     - aces:
#       - destination:
#           address: 192.0.3.0
#           port_protocol:
#             eq: www
#           wildcard_bits: 0.0.0.255
#         grant: deny
#         option:
#           traceroute: true
#         protocol: tcp
#         protocol_options:
#           tcp:
#             fin: true
#         sequence: 10
#         source:
#           address: 192.0.2.0
#           wildcard_bits: 0.0.0.255
#         ttl:
#           eq: 10
#       acl_type: extended
#       name: test
#     afi: ipv4
#   - acls:
#     - aces:
#       - destination:
#           any: true
#           port_protocol:
#             eq: telnet
#         dscp: af11
#         grant: deny
#         protocol: tcp
#         protocol_options:
#           tcp:
#             ack: true
#         sequence: 10
#         source:
#           any: true
#           port_protocol:
#             eq: www
#       name: R1_TRAFFIC
#     afi: ipv6
#   invocation:
#     module_args:
#       config: null
#       running_config: null
#       state: gathered

# TASK [Setting host facts for pop_ace filter plugin] *************************
# ok: [xe_machine] => changed=false
#   ansible_facts:
#     acls_facts:
#     - acls:
#       - aces:
#         - destination:
#             address: 192.0.3.0
#             wildcard_bits: 0.0.0.255
#           dscp: ef
#           grant: deny
#           protocol: icmp
#           protocol_options:
#             icmp:
#               traceroute: true
#           sequence: 10
#           source:
#             address: 192.0.2.0
#             wildcard_bits: 0.0.0.255
#           ttl:
#             eq: 10
#         - destination:
#             host: 198.51.110.0
#             port_protocol:
#               eq: telnet
#           grant: deny
#           protocol: tcp
#           protocol_options:
#             tcp:
#               ack: true
#           sequence: 20
#           source:
#             host: 198.51.100.0
#         acl_type: extended
#         name: '110'
#       - aces:
#         - destination:
#             address: 198.51.101.0
#             port_protocol:
#               eq: telnet
#             wildcard_bits: 0.0.0.255
#           grant: deny
#           protocol: tcp
#           protocol_options:
#             tcp:
#               ack: true
#           sequence: 10
#           source:
#             address: 198.51.100.0
#             wildcard_bits: 0.0.0.255
#           tos:
#             service_value: 12
#         - destination:
#             address: 192.0.4.0
#             port_protocol:
#               eq: www
#             wildcard_bits: 0.0.0.255
#           dscp: ef
#           grant: deny
#           protocol: tcp
#           protocol_options:
#             tcp:
#               ack: true
#           sequence: 20
#           source:
#             address: 192.0.3.0
#             wildcard_bits: 0.0.0.255
#           ttl:
#             lt: 20
#         acl_type: extended
#         name: '123'
#       - aces:
#         - grant: deny
#           sequence: 10
#           source:
#             host: 192.168.1.200
#         - grant: deny
#           sequence: 20
#           source:
#             address: 192.168.2.0
#             wildcard_bits: 0.0.0.255
#         acl_type: standard
#         name: std_acl
#       - aces:
#         - destination:
#             address: 192.0.3.0
#             port_protocol:
#               eq: www
#             wildcard_bits: 0.0.0.255
#           grant: deny
#           option:
#             traceroute: true
#           protocol: tcp
#           protocol_options:
#             tcp:
#               fin: true
#           sequence: 10
#           source:
#             address: 192.0.2.0
#             wildcard_bits: 0.0.0.255
#           ttl:
#             eq: 10
#         acl_type: extended
#         name: test
#       afi: ipv4
#     - acls:
#       - aces:
#         - destination:
#             any: true
#             port_protocol:
#               eq: telnet
#           dscp: af11
#           grant: deny
#           protocol: tcp
#           protocol_options:
#             tcp:
#               ack: true
#           sequence: 10
#           source:
#             any: true
#             port_protocol:
#               eq: www
#         name: R1_TRAFFIC
#       afi: ipv6
#     filter_options:
#       match_all: true
#     match_criteria:
#       afi: ipv4
#       destination: 192.0.3.0
#       source: 192.0.2.0

# TASK [Invoke pop_ace filter plugin] *****************************************
# ok: [xe_machine] => changed=false
#   ansible_facts:
#     clean_acls:
#       clean_acls:
#         acls:
#         - acls:
#           - aces:
#             - destination:
#                 host: 198.51.110.0
#                 port_protocol:
#                   eq: telnet
#               grant: deny
#               protocol: tcp
#               protocol_options:
#                 tcp:
#                   ack: true
#               sequence: 20
#               source:
#                 host: 198.51.100.0
#             name: '110'
#           - aces:
#             - destination:
#                 address: 198.51.101.0
#                 port_protocol:
#                   eq: telnet
#                 wildcard_bits: 0.0.0.255
#               grant: deny
#               protocol: tcp
#               protocol_options:
#                 tcp:
#                   ack: true
#               sequence: 10
#               source:
#                 address: 198.51.100.0
#                 wildcard_bits: 0.0.0.255
#               tos:
#                 service_value: 12
#             - destination:
#                 address: 192.0.4.0
#                 port_protocol:
#                   eq: www
#                 wildcard_bits: 0.0.0.255
#               dscp: ef
#               grant: deny
#               protocol: tcp
#               protocol_options:
#                 tcp:
#                   ack: true
#               sequence: 20
#               source:
#                 address: 192.0.3.0
#                 wildcard_bits: 0.0.0.255
#               ttl:
#                 lt: 20
#             name: '123'
#           - aces:
#             - grant: deny
#               sequence: 10
#               source:
#                 host: 192.168.1.200
#             - grant: deny
#               sequence: 20
#               source:
#                 address: 192.168.2.0
#                 wildcard_bits: 0.0.0.255
#             name: std_acl
#           afi: ipv4
#         - acls:
#           - aces:
#             - destination:
#                 any: true
#                 port_protocol:
#                   eq: telnet
#               dscp: af11
#               grant: deny
#               protocol: tcp
#               protocol_options:
#                 tcp:
#                   ack: true
#               sequence: 10
#               source:
#                 any: true
#                 port_protocol:
#                   eq: www
#             name: R1_TRAFFIC
#           afi: ipv6
#       removed_aces:
#         acls:
#         - acls:
#           - aces:
#             - destination:
#                 address: 192.0.3.0
#                 wildcard_bits: 0.0.0.255
#               dscp: ef
#               grant: deny
#               protocol: icmp
#               protocol_options:
#                 icmp:
#                   traceroute: true
#               sequence: 10
#               source:
#                 address: 192.0.2.0
#                 wildcard_bits: 0.0.0.255
#               ttl:
#                 eq: 10
#             name: '110'
#           - aces:
#             - destination:
#                 address: 192.0.3.0
#                 port_protocol:
#                   eq: www
#                 wildcard_bits: 0.0.0.255
#               grant: deny
#               option:
#                 traceroute: true
#               protocol: tcp
#               protocol_options:
#                 tcp:
#                   fin: true
#               sequence: 10
#               source:
#                 address: 192.0.2.0
#                 wildcard_bits: 0.0.0.255
#               ttl:
#                 eq: 10
#             name: test
#           afi: ipv4
#         - acls: []
#           afi: ipv6

# TASK [Override ACLs config with device existing ACLs config] *******************
# changed: [xe_machine] => changed=true
#   after:
#   - acls:
#     - aces:
#       - destination:
#           host: 198.51.110.0
#           port_protocol:
#             eq: telnet
#         grant: deny
#         protocol: tcp
#         protocol_options:
#           tcp:
#             ack: true
#         sequence: 20
#         source:
#           host: 198.51.100.0
#       acl_type: extended
#       name: '110'
#     - aces:
#       - destination:
#           address: 198.51.101.0
#           port_protocol:
#             eq: telnet
#           wildcard_bits: 0.0.0.255
#         grant: deny
#         protocol: tcp
#         protocol_options:
#           tcp:
#             ack: true
#         sequence: 10
#         source:
#           address: 198.51.100.0
#           wildcard_bits: 0.0.0.255
#         tos:
#           service_value: 12
#       - destination:
#           address: 192.0.4.0
#           port_protocol:
#             eq: www
#           wildcard_bits: 0.0.0.255
#         dscp: ef
#         grant: deny
#         protocol: tcp
#         protocol_options:
#           tcp:
#             ack: true
#         sequence: 20
#         source:
#           address: 192.0.3.0
#           wildcard_bits: 0.0.0.255
#         ttl:
#           lt: 20
#       acl_type: extended
#       name: '123'
#     - aces:
#       - grant: deny
#         sequence: 10
#         source:
#           host: 192.168.1.200
#       - grant: deny
#         sequence: 20
#         source:
#           address: 192.168.2.0
#           wildcard_bits: 0.0.0.255
#       acl_type: standard
#       name: std_acl
#     afi: ipv4
#   - acls:
#     - aces:
#       - destination:
#           any: true
#           port_protocol:
#             eq: telnet
#         dscp: af11
#         grant: deny
#         protocol: tcp
#         protocol_options:
#           tcp:
#             ack: true
#         sequence: 10
#         source:
#           any: true
#           port_protocol:
#             eq: www
#       name: R1_TRAFFIC
#     afi: ipv6
#   before:
#   - acls:
#     - aces:
#       - destination:
#           address: 192.0.3.0
#           wildcard_bits: 0.0.0.255
#         dscp: ef
#         grant: deny
#         protocol: icmp
#         protocol_options:
#           icmp:
#             traceroute: true
#         sequence: 10
#         source:
#           address: 192.0.2.0
#           wildcard_bits: 0.0.0.255
#         ttl:
#           eq: 10
#       - destination:
#           host: 198.51.110.0
#           port_protocol:
#             eq: telnet
#         grant: deny
#         protocol: tcp
#         protocol_options:
#           tcp:
#             ack: true
#         sequence: 20
#         source:
#           host: 198.51.100.0
#       acl_type: extended
#       name: '110'
#     - aces:
#       - destination:
#           address: 198.51.101.0
#           port_protocol:
#             eq: telnet
#           wildcard_bits: 0.0.0.255
#         grant: deny
#         protocol: tcp
#         protocol_options:
#           tcp:
#             ack: true
#         sequence: 10
#         source:
#           address: 198.51.100.0
#           wildcard_bits: 0.0.0.255
#         tos:
#           service_value: 12
#       - destination:
#           address: 192.0.4.0
#           port_protocol:
#             eq: www
#           wildcard_bits: 0.0.0.255
#         dscp: ef
#         grant: deny
#         protocol: tcp
#         protocol_options:
#           tcp:
#             ack: true
#         sequence: 20
#         source:
#           address: 192.0.3.0
#           wildcard_bits: 0.0.0.255
#         ttl:
#           lt: 20
#       acl_type: extended
#       name: '123'
#     - aces:
#       - grant: deny
#         sequence: 10
#         source:
#           host: 192.168.1.200
#       - grant: deny
#         sequence: 20
#         source:
#           address: 192.168.2.0
#           wildcard_bits: 0.0.0.255
#       acl_type: standard
#       name: std_acl
#     - aces:
#       - destination:
#           address: 192.0.3.0
#           port_protocol:
#             eq: www
#           wildcard_bits: 0.0.0.255
#         grant: deny
#         option:
#           traceroute: true
#         protocol: tcp
#         protocol_options:
#           tcp:
#             fin: true
#         sequence: 10
#         source:
#           address: 192.0.2.0
#           wildcard_bits: 0.0.0.255
#         ttl:
#           eq: 10
#       acl_type: extended
#       name: test
#     afi: ipv4
#   - acls:
#     - aces:
#       - destination:
#           any: true
#           port_protocol:
#             eq: telnet
#         dscp: af11
#         grant: deny
#         protocol: tcp
#         protocol_options:
#           tcp:
#             ack: true
#         sequence: 10
#         source:
#           any: true
#           port_protocol:
#             eq: www
#       name: R1_TRAFFIC
#     afi: ipv6
#   commands:
#   - ip access-list extended 110
#   - no 10 deny icmp 192.0.2.0 0.0.0.255 192.0.3.0 0.0.0.255 traceroute dscp ef ttl eq 10
#   - no ip access-list extended test

"""

from ansible.errors import AnsibleFilterError
from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)

from ansible_collections.ansible.netcommon.plugins.plugin_utils.pop_ace import pop_ace


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment


@pass_environment
def _pop_ace(*args, **kwargs):
    """remove ace entries from a acl data"""

    keys = ["data", "filter_options", "match_criteria"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="pop_ace")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return pop_ace(**updated_data)


class FilterModule(object):
    """pop_ace"""

    def filters(self):
        """a mapping of filter names to functions"""
        return {"pop_ace": _pop_ace}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: devices_wireless_connection_stats_info
short_description: Information module for devices _wireless _connectionstats
description:
- Get all devices _wireless _connectionstats.
- Aggregated connectivity info for a given AP on this network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  serial:
    description:
    - Serial path parameter.
    type: str
  t0:
    description:
    - T0 query parameter. The beginning of the timespan for the data. The maximum lookback period is 180 days from today.
    type: str
  t1:
    description:
    - T1 query parameter. The end of the timespan for the data. T1 can be a maximum of 7 days after t0.
    type: str
  timespan:
    description:
    - >
      Timespan query parameter. The timespan for which the information will be fetched. If specifying timespan, do
      not specify parameters t0 and t1. The value must be in seconds and be less than or equal to 7 days.
    type: float
  band:
    description:
    - >
      Band query parameter. Filter results by band (either '2.4', '5' or '6'). Note that data prior to February
      2020 will not have band information.
    type: str
  ssid:
    description:
    - Ssid query parameter. Filter results by SSID.
    type: int
  vlan:
    description:
    - Vlan query parameter. Filter results by VLAN.
    type: int
  apTag:
    description:
    - ApTag query parameter. Filter results by AP Tag.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for wireless getDeviceWirelessConnectionStats
  description: Complete reference of the getDeviceWirelessConnectionStats API.
  link: https://developer.cisco.com/meraki/api-v1/#!get-device-wireless-connection-stats
notes:
  - SDK Method used are
    wireless.Wireless.get_device_wireless_connection_stats,

  - Paths used are
    get /devices/{serial}/wireless/connectionStats,
"""

EXAMPLES = r"""
- name: Get all devices _wireless _connectionstats
  cisco.meraki.devices_wireless_connection_stats_info:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    t0: string
    t1: string
    timespan: 0
    band: string
    ssid: 0
    vlan: 0
    apTag: string
    serial: string
  register: result

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "serial": "string",
      "connectionStats": {
        "assoc": 0,
        "auth": 0,
        "dhcp": 0,
        "dns": 0,
        "success": 0
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: organizations_switch_ports_by_switch_info
short_description: Information module for organizations _switch _ports _byswitch
description:
- Get all organizations _switch _ports _byswitch.
- List the switchports in an organization by switch.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
    - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
    - PerPage query parameter. The number of entries per page returned. Acceptable range is 3 - 50. Default is 50.
    type: int
  startingAfter:
    description:
    - >
      StartingAfter query parameter. A token used by the server to indicate the start of the page. Often this is a
      timestamp or an ID but it is not limited to those. This parameter should not be defined by client
      applications. The link for the first, last, prev, or next page in the HTTP Link header should define it.
    type: str
  endingBefore:
    description:
    - >
      EndingBefore query parameter. A token used by the server to indicate the end of the page. Often this is a
      timestamp or an ID but it is not limited to those. This parameter should not be defined by client
      applications. The link for the first, last, prev, or next page in the HTTP Link header should define it.
    type: str
  networkIds:
    description:
    - NetworkIds query parameter. Optional parameter to filter switchports by network.
    elements: str
    type: list
  portProfileIds:
    description:
    - PortProfileIds query parameter. Optional parameter to filter switchports belonging to the specified port profiles.
    elements: str
    type: list
  name:
    description:
    - >
      Name query parameter. Optional parameter to filter switchports belonging to switches by name. All returned
      switches will have a name that contains the search term or is an exact match.
    type: str
  mac:
    description:
    - >
      Mac query parameter. Optional parameter to filter switchports belonging to switches by MAC address. All
      returned switches will have a MAC address that contains the search term or is an exact match.
    type: str
  macs:
    description:
    - >
      Macs query parameter. Optional parameter to filter switchports by one or more MAC addresses belonging to
      devices. All switchports returned belong to MAC addresses of switches that are an exact match.
    elements: str
    type: list
  serial:
    description:
    - >
      Serial query parameter. Optional parameter to filter switchports belonging to switches by serial number. All
      returned switches will have a serial number that contains the search term or is an exact match.
    type: str
  serials:
    description:
    - >
      Serials query parameter. Optional parameter to filter switchports belonging to switches with one or more
      serial numbers. All switchports returned belong to serial numbers of switches that are an exact match.
    elements: str
    type: list
  configurationUpdatedAfter:
    description:
    - >
      ConfigurationUpdatedAfter query parameter. Optional parameter to filter results by switches where the
      configuration has been updated after the given timestamp.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for switch getOrganizationSwitchPortsBySwitch
  description: Complete reference of the getOrganizationSwitchPortsBySwitch API.
  link: https://developer.cisco.com/meraki/api-v1/#!get-organization-switch-ports-by-switch
notes:
  - SDK Method used are
    switch.Switch.get_organization_switch_ports_by_switch,

  - Paths used are
    get /organizations/{organizationId}/switch/ports/bySwitch,
"""

EXAMPLES = r"""
- name: Get all organizations _switch _ports _byswitch
  cisco.meraki.organizations_switch_ports_by_switch_info:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkIds: []
    portProfileIds: []
    name: string
    mac: string
    macs: []
    serial: string
    serials: []
    configurationUpdatedAfter: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "name": "string",
        "serial": "string",
        "mac": "string",
        "network": {
          "name": "string",
          "id": "string"
        },
        "model": "string",
        "ports": [
          {
            "portId": "string",
            "name": "string",
            "tags": [
              "string"
            ],
            "enabled": true,
            "poeEnabled": true,
            "type": "string",
            "vlan": 0,
            "voiceVlan": 0,
            "allowedVlans": "string",
            "rstpEnabled": true,
            "stpGuard": "string",
            "linkNegotiation": "string",
            "accessPolicyType": "string",
            "stickyMacAllowList": [
              "string"
            ],
            "stickyMacAllowListLimit": 0
          }
        ]
      }
    ]
"""
